/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.VirtualFileSystem.FTP;

import java.util.HashMap;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Listener.JSListener;
import com.sos.JSHelper.Options.JSJobChain;
import com.sos.JSHelper.Options.JSOptionsClass;
import com.sos.JSHelper.Options.SOSOptionAuthenticationMethod;
import com.sos.JSHelper.Options.SOSOptionBoolean;
import com.sos.JSHelper.Options.SOSOptionCommandScript;
import com.sos.JSHelper.Options.SOSOptionCommandScriptFile;
import com.sos.JSHelper.Options.SOSOptionCommandString;
import com.sos.JSHelper.Options.SOSOptionFileName;
import com.sos.JSHelper.Options.SOSOptionFolderName;
import com.sos.JSHelper.Options.SOSOptionHostName;
import com.sos.JSHelper.Options.SOSOptionInFileName;
import com.sos.JSHelper.Options.SOSOptionIniFileName;
import com.sos.JSHelper.Options.SOSOptionInteger;
import com.sos.JSHelper.Options.SOSOptionJadeOperation;
import com.sos.JSHelper.Options.SOSOptionOutFileName;
import com.sos.JSHelper.Options.SOSOptionPassword;
import com.sos.JSHelper.Options.SOSOptionPortNumber;
import com.sos.JSHelper.Options.SOSOptionProcessID;
import com.sos.JSHelper.Options.SOSOptionRegExp;
import com.sos.JSHelper.Options.SOSOptionString;
import com.sos.JSHelper.Options.SOSOptionStringValueList;
import com.sos.JSHelper.Options.SOSOptionTransferMode;
import com.sos.JSHelper.Options.SOSOptionTransferType;
import com.sos.JSHelper.Options.SOSOptionUserName;
import com.sos.VirtualFileSystem.Interfaces.ISOSAuthenticationOptions;
import com.sos.VirtualFileSystem.Interfaces.ISOSConnectionOptions;
import com.sos.VirtualFileSystem.Interfaces.ISOSFtpOptions;
import com.sos.i18n.annotation.I18NResourceBundle;
import com.sos.localization.Messages;

/**
* \class 		SOSFtpOptionsSuperClass - Transfer files by FTP/SFTP and execute commands by SSH
*
* \brief An Options-Class with all Options.
*
* \verbatim ;
* mechanicaly created by JobDocu2OptionsClass.xslt from http://www.sos-berlin.com at 2010-05-15-14-28-00 
* \endverbatim
* \section OptionsTable Tabelle der vorhandenen Optionen
* 
* Tabelle mit allen Optionen
* 
* MethodName
* Title
* Setting
* Description
* IsMandatory
* DataType
* InitialValue
* TestValue
* 
* 
*
* \section TestData Eine Hilfe zum Erzeugen einer HashMap mit Testdaten
*
* Die folgenden Methode kann verwendet werden, um fr einen Test eine HashMap
* mit sinnvollen Werten fr die einzelnen Optionen zu erzeugen.
*
* \verbatim
private HashMap <String, String> SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM) {
pobjHM.put ("		SOSFtpOptionsSuperClass.auth_file", "test");  // This parameter specifies the path and name of a user's pr
return pobjHM;
}  //  private void SetJobSchedulerSSHJobOptions (HashMap <String, String> pobjHM)
* \endverbatim
*/
@JSOptionClass(name = "SOSFtpOptionsSuperClass", description = "SOSFtpOptionsSuperClass")
@I18NResourceBundle(baseName = "SOSVirtualFileSystem", defaultLocale = "en")
public class SOSFtpOptionsSuperClass extends JSOptionsClass implements ISOSConnectionOptions, ISOSAuthenticationOptions, ISOSFtpOptions {

	protected Messages				objMsg				= new Messages(this.getClass().getAnnotation(I18NResourceBundle.class).baseName());

	/**
	 * 
	 */
	private static final long	serialVersionUID	= -4445655877481869778L;

	private final String		conClassName		= "SOSFtpOptionsSuperClass";
	@JSOptionDefinition(name = "source_dir", description = "Optional account info for authentication with an", key = "account", type = "SOSOptionString", mandatory = false)
	public SOSOptionFolderName	SourceDir			= new SOSOptionFolderName(this, conClassName + ".source_dir", // HashMap-Key
															"local_dir Local directory into which or from which", // Titel
															"", // InitValue
															"", // DefaultValue
															false // isMandatory
													);
	@JSOptionDefinition(name = "target_dir", description = "Optional account info for authentication with an", key = "account", type = "SOSOptionString", mandatory = false)
	public SOSOptionFolderName	TargetDir			= new SOSOptionFolderName(this, conClassName + ".target_dir", // HashMap-Key
															"target_dir directory into which or from which", // Titel
															"", // InitValue
															"", // DefaultValue
															false // isMandatory
													);

	/**
	* \var account : Optional account info for authentication with an
	*
	Optional account info for authentication with an (FTP) server.
	*
	*/
	@JSOptionDefinition(name = "account", description = "Optional account info for authentication with an", key = "account", type = "SOSOptionString", mandatory = false)
	public SOSOptionString		account				= new SOSOptionString(this, conClassName + ".account", // HashMap-Key
															"Optional account info for authentication with an", // Titel
															" ", // InitValue
															" ", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief getaccount
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getaccount() {
		return account;
	}

	/**
	 * \brief setaccount
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_account
	 */
	public void setaccount(SOSOptionString p_account) {
		this.account = p_account;
	}

	/**
	* \var alternative_account : Alternative parameter for the primary parameter
	*
	Alternative parameter for the primary parameter account. The alternative parameters are used solely should the connection to an FTP/SFTP server fail, e.g. if the server were not available or if invalid credentials were used.
	*
	*/
	@JSOptionDefinition(name = "alternative_account", description = "Alternative parameter for the primary parameter", key = "alternative_account", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	alternative_account	= new SOSOptionString(this, conClassName + ".alternative_account", // HashMap-Key
														"Alternative parameter for the primary parameter", // Titel
														" ", // InitValue
														" ", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getalternative_account
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getalternative_account() {
		return alternative_account;
	}

	/**
	 * \brief setalternative_account
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_alternative_account
	 */
	public void setalternative_account(SOSOptionString p_alternative_account) {
		this.alternative_account = p_alternative_account;
	}

	/**
	* \var alternative_host : Alternative parameter for the primary parameter
	*
	Alternative parameter for the primary parameter host. The alternative parameters are used solely should the connection to an FTP/SFTP server fail, e.g. if the server were not available or if invalid credentials were used.
	*
	*/
	@JSOptionDefinition(name = "alternative_host", description = "Alternative parameter for the primary parameter", key = "alternative_host", type = "SOSOptionHostName", mandatory = false)
	public SOSOptionHostName	alternative_host	= new SOSOptionHostName(this, conClassName + ".alternative_host", // HashMap-Key
															"Alternative parameter for the primary parameter", // Titel
															" ", // InitValue
															" ", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief getalternative_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionHostName getalternative_host() {
		return alternative_host;
	}

	/**
	 * \brief setalternative_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_alternative_host
	 */
	public void setalternative_host(SOSOptionHostName p_alternative_host) {
		this.alternative_host = p_alternative_host;
	}

	/**
	* \var alternative_passive_mode : Alternative parameter for the primary parameter
	*
	Alternative parameter for the primary parameter passive_mode. The alternative parameters are used solely should the connection to an FTP/SFTP server fail, e.g. if the server were not available or if invalid credentials were used.
	*
	*/
	@JSOptionDefinition(name = "alternative_passive_mode", description = "Alternative parameter for the primary parameter", key = "alternative_passive_mode", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	alternative_passive_mode	= new SOSOptionString(this, conClassName + ".alternative_passive_mode", // HashMap-Key
																"Alternative parameter for the primary parameter", // Titel
																" ", // InitValue
																" ", // DefaultValue
																false // isMandatory
														);

	/**
	 * \brief getalternative_passive_mode
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getalternative_passive_mode() {
		return alternative_passive_mode;
	}

	/**
	 * \brief setalternative_passive_mode
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_alternative_passive_mode
	 */
	public void setalternative_passive_mode(SOSOptionString p_alternative_passive_mode) {
		this.alternative_passive_mode = p_alternative_passive_mode;
	}

	/**
	* \var alternative_password : Alternative parameter for the primary parameter
	*
	Alternative parameter for the primary parameter password. The alternative parameters are used solely should the connection to an FTP/SFTP server fail, e.g. if the server were not available or if invalid credentials were used.
	*
	*/
	@JSOptionDefinition(name = "alternative_password", description = "Alternative parameter for the primary parameter", key = "alternative_password", type = "SOSOptionString", mandatory = false)
	public SOSOptionPassword	alternative_password	= new SOSOptionPassword(this, conClassName + ".alternative_password", // HashMap-Key
																"Alternative parameter for the primary parameter", // Titel
																" ", // InitValue
																" ", // DefaultValue
																false // isMandatory
														);

	/**
	 * \brief getalternative_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionPassword getalternative_password() {
		return alternative_password;
	}

	/**
	 * \brief setalternative_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_alternative_password
	 */
	public void setalternative_password(SOSOptionPassword p_alternative_password) {
		this.alternative_password = p_alternative_password;
	}

	/**
	* \var alternative_port : Alternative parameter for the primary parameter
	*
	Alternative parameter for the primary parameter port. The alternative parameters are used solely should the connection to an FTP/SFTP server fail, e.g. if the server were not available or if invalid credentials were used.
	*
	*/
	@JSOptionDefinition(name = "alternative_port", description = "Alternative parameter for the primary parameter", key = "alternative_port", type = "SOSOptionPortNumber", mandatory = false)
	public SOSOptionPortNumber	alternative_port	= new SOSOptionPortNumber(this, conClassName + ".alternative_port", // HashMap-Key
															"Alternative parameter for the primary parameter", // Titel
															"21", // InitValue
															"21", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief getalternative_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionPortNumber getalternative_port() {
		return alternative_port;
	}

	/**
	 * \brief setalternative_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_alternative_port
	 */
	public void setalternative_port(SOSOptionPortNumber p_alternative_port) {
		this.alternative_port = p_alternative_port;
	}

	public SOSOptionPortNumber	AlternativePortNumber	= (SOSOptionPortNumber) alternative_port.SetAlias(conClassName + ".AlternativePortNumber");

	/**
	* \var alternative_remote_dir : Alternative parameter for the primary parameter
	*
	Alternative parameter for the primary parameter remote_dir. The alternative parameters are used solely should the connection to an FTP/SFTP server fail, e.g. if the server were not available or if invalid credentials were used.
	*
	*/
	@JSOptionDefinition(name = "alternative_remote_dir", description = "Alternative parameter for the primary parameter", key = "alternative_remote_dir", type = "SOSOptionString", mandatory = false)
	public SOSOptionString		alternative_remote_dir	= new SOSOptionString(this, conClassName + ".alternative_remote_dir", // HashMap-Key
																"Alternative parameter for the primary parameter", // Titel
																" ", // InitValue
																" ", // DefaultValue
																false // isMandatory
														);

	/**
	 * \brief getalternative_remote_dir
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getalternative_remote_dir() {
		return alternative_remote_dir;
	}

	/**
	 * \brief setalternative_remote_dir
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_alternative_remote_dir
	 */
	public void setalternative_remote_dir(SOSOptionString p_alternative_remote_dir) {
		this.alternative_remote_dir = p_alternative_remote_dir;
	}

	/**
	* \var alternative_transfer_mode : Alternative parameter for the primary parameter
	*
	Alternative parameter for the primary parameter transfer_mode. The alternative parameters are used solely should the connection to an FTP/SFTP server fail, e.g. if the server were not available or if invalid credentials were used.
	*
	*/
	@JSOptionDefinition(name = "alternative_transfer_mode", description = "Alternative parameter for the primary parameter", key = "alternative_transfer_mode", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	alternative_transfer_mode	= new SOSOptionString(this, conClassName + ".alternative_transfer_mode", // HashMap-Key
																"Alternative parameter for the primary parameter", // Titel
																" ", // InitValue
																" ", // DefaultValue
																false // isMandatory
														);

	/**
	 * \brief getalternative_transfer_mode
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getalternative_transfer_mode() {
		return alternative_transfer_mode;
	}

	/**
	 * \brief setalternative_transfer_mode
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_alternative_transfer_mode
	 */
	public void setalternative_transfer_mode(SOSOptionString p_alternative_transfer_mode) {
		this.alternative_transfer_mode = p_alternative_transfer_mode;
	}

	/**
	* \var alternative_user : Alternative parameter for the primary parameter
	*
	Alternative parameter for the primary parameter user. The alternative parameters are used solely should the connection to an FTP/SFTP server fail, e.g. if the server were not available or if invalid credentials were used.
	*
	*/
	@JSOptionDefinition(name = "alternative_user", description = "Alternative parameter for the primary parameter", key = "alternative_user", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	alternative_user	= new SOSOptionString(this, conClassName + ".alternative_user", // HashMap-Key
														"Alternative parameter for the primary parameter", // Titel
														" ", // InitValue
														" ", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getalternative_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getalternative_user() {
		return alternative_user;
	}

	/**
	 * \brief setalternative_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_alternative_user
	 */
	public void setalternative_user(SOSOptionString p_alternative_user) {
		this.alternative_user = p_alternative_user;
	}

	/**
	* \var append_files : This parameter specifies whether the content of a
	*
	This parameter specifies whether the content of a source file should be appended to the target file should this file exist. The parameter overwrite_files will be ignored if this parameter is specified with the value true.
	*
	*/
	@JSOptionDefinition(name = "append_files", description = "This parameter specifies whether the content of a", key = "append_files", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	append_files	= new SOSOptionBoolean(this, conClassName + ".append_files", // HashMap-Key
													"This parameter specifies whether the content of a", // Titel
													"false", // InitValue
													"false", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getappend_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getappend_files() {
		return append_files;
	}

	/**
	 * \brief setappend_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_append_files
	 */
	public void setappend_files(SOSOptionBoolean p_append_files) {
		this.append_files = p_append_files;
	}

	/**
	* \var atomic_prefix : This parameter specifies whether target files shou
	*
	This parameter specifies whether target files should be created with a prefix such as "~", and must be renamed to the target file name after the file transfer is completed without errors. This mechanism is useful if the target directory is monitored for incoming files by some application and if files are required to appear atomically instead of being subsequently written to. The temporary prefix is specified as the value of this parameter. This setting is recommended should target directories be monitored by an application or a JobScheduler.
	*
	*/
	@JSOptionDefinition(name = "atomic_prefix", description = "This parameter specifies whether target files shou", key = "atomic_prefix", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	atomic_prefix	= new SOSOptionString(this, conClassName + ".atomic_prefix", // HashMap-Key
													"This parameter specifies whether target files shou", // Titel
													"", // InitValue
													"", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getatomic_prefix
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getatomic_prefix() {
		return atomic_prefix;
	}

	/**
	 * \brief setatomic_prefix
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_atomic_prefix
	 */
	public void setatomic_prefix(SOSOptionString p_atomic_prefix) {
		this.atomic_prefix = p_atomic_prefix;
	}

	/**
	* \var atomic_suffix : This parameter specifies whether target files shou
	*
	This parameter specifies whether target files should be created with a suffix such as "~", and should be renamed to the target file name after the file transfer is completed. This mechanism is useful if the target directory is monitored for incoming files by some application and if files are required to appear atomically instead of being subsequently written to. The temporary suffix is specified as the value of this parameter. This setting is recommended should target directories be monitored by an application or a Job Scheduler.
	*
	*/
	@JSOptionDefinition(name = "atomic_suffix", description = "This parameter specifies whether target files shou", key = "atomic_suffix", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	atomic_suffix	= new SOSOptionString(this, conClassName + ".atomic_suffix", // HashMap-Key
													"This parameter specifies whether target files shou", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getatomic_suffix
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getatomic_suffix() {
		return atomic_suffix;
	}

	/**
	 * \brief setatomic_suffix
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_atomic_suffix
	 */
	public void setatomic_suffix(SOSOptionString p_atomic_suffix) {
		this.atomic_suffix = p_atomic_suffix;
	}

	/**
	* \var banner_footer : Name der Template-Datei fr Protokoll-Ende This p
	*
	This program logs output to stdout or to a file that has been specified by the parameter log_filename. A template can be used in order to organize the output that is created. The output is grouped into header, file list and footer. This parameter specifies a template file for footer output. Templates can use internal variables and parameters as placeholders in the form %{placeholder}. The standard footer template looks like this: ************************************************************************* execution status = %{status} successful transfers = %{successful_transfers} failed transfers = %{failed_transfers} last error = %{last_error} *************************************************************************
	*
	*/
	@JSOptionDefinition(name = "banner_footer", description = "Name der Template-Datei fr Protokoll-Ende This p", key = "banner_footer", type = "SOSOptionInFileName", mandatory = false)
	public SOSOptionInFileName	banner_footer	= new SOSOptionInFileName(this, conClassName + ".banner_footer", // HashMap-Key
														"Name der Template-Datei fr Protokoll-Ende This p", // Titel
														"", // InitValue
														"", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getbanner_footer
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInFileName getbanner_footer() {
		return banner_footer;
	}

	/**
	 * \brief setbanner_footer
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_banner_footer
	 */
	public void setbanner_footer(SOSOptionInFileName p_banner_footer) {
		this.banner_footer = p_banner_footer;
	}

	/**
	* \var banner_header : Name of Template-File for log-File
	*
	This program logs output to stdout or to a file that has been specified by the parameter log_filename. A template can be used in order to organize the output that is created. The output is grouped into header, file list and footer. This parameter specifies a template file for header output. Templates can use internal variables and parameters as placeholders in the form %{placeholder}. The standard header template looks like this: ************************************************************************* * * * SOSFTP - Managed File Transfer Utility * * -------------------------------------- * * * ************************************************************************* version = %{version} date = %{date} %{time} operation = %{operation} protocol = %{protocol} file specification = %{file_spec} file path = %{file_path} source host = %{localhost} (%{local_host_ip}) local directory = %{local_dir} jump host = %{jump_host} target host = %{host} (%{host_ip}) target directory = %{remote_dir} pid = %{current_pid} ppid = %{ppid} *************************************************************************
	*
	*/
	@JSOptionDefinition(name = "banner_header", description = "Name of Template-File for log-File", key = "banner_header", type = "SOSOptionInFileName", mandatory = false)
	public SOSOptionInFileName	banner_header	= new SOSOptionInFileName(this, conClassName + ".banner_header", // HashMap-Key
														"Name of Template-File for log-File", // Titel
														"", // InitValue
														"", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getbanner_header
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInFileName getbanner_header() {
		return banner_header;
	}

	/**
	 * \brief setbanner_header
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_banner_header
	 */
	public void setbanner_header(SOSOptionInFileName p_banner_header) {
		this.banner_header = p_banner_header;
	}

	/**
	* \var check_interval : This parameter specifies the interval in seconds
	*
	This parameter specifies the interval in seconds between two file transfer trials, if repeated transfer of files has been configured using the check_retry parameter.
	*
	*/
	@JSOptionDefinition(name = "check_interval", description = "This parameter specifies the interval in seconds", key = "check_interval", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger	check_interval	= new SOSOptionInteger(this, conClassName + ".check_interval", // HashMap-Key
													"This parameter specifies the interval in seconds", // Titel
													"60", // InitValue
													"60", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getcheck_interval
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger getcheck_interval() {
		return check_interval;
	}

	/**
	 * \brief setcheck_interval
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_check_interval
	 */
	public void setcheck_interval(SOSOptionInteger p_check_interval) {
		this.check_interval = p_check_interval;
	}

	/**
	* \var check_retry : This parameter specifies whether a file transfer
	*
	This parameter specifies whether a file transfer should be repeated in order to ensure that the file was complete when the transfer started. This is relevant for Unix systems that allow read and write access to a file at the same time. This parameter causes the size of the current file transfer and of the previous file transfer to be compared and repeats transferring one file up to the number of trials specified by this parameter. Should the file size of both transfers be the same, then it is assumed that the file was complete at the FTP/SFTP server. The interval between two trials to transfer a file is configured using the check_interval parameter.
	*
	*/
	@JSOptionDefinition(name = "check_retry", description = "This parameter specifies whether a file transfer", key = "check_retry", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger	check_retry	= new SOSOptionInteger(this, conClassName + ".check_retry", // HashMap-Key
												"This parameter specifies whether a file transfer", // Titel
												"0", // InitValue
												"0", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getcheck_retry
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger getcheck_retry() {
		return check_retry;
	}

	/**
	 * \brief setcheck_retry
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_check_retry
	 */
	public void setcheck_retry(SOSOptionInteger p_check_retry) {
		this.check_retry = p_check_retry;
	}

	/**
	* \var check_size : This parameter determines whether the original f
	*
	This parameter determines whether the original file size and the number of bytes transferred should be compared after a file transfer and whether an error should be raised if they would not match.
	*
	*/
	@JSOptionDefinition(name = "check_size", description = "This parameter determines whether the original f", key = "check_size", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	check_size	= new SOSOptionBoolean(this, conClassName + ".check_size", // HashMap-Key
												"This parameter determines whether the original f", // Titel
												"true", // InitValue
												"true", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getcheck_size
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getcheck_size() {
		return check_size;
	}

	/**
	 * \brief setcheck_size
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_check_size
	 */
	public void setcheck_size(SOSOptionBoolean p_check_size) {
		this.check_size = p_check_size;
	}

	public SOSOptionBoolean		CheckFileSizeAfterTransfer	= (SOSOptionBoolean) check_size.SetAlias(conClassName + ".CheckFileSizeAfterTransfer");

	/**
	* \var classpath_base : The parameter is used during installation of this
	*
	The parameter is used during installation of this program on a remote server with the parameter operation=installcode>. This parameter specifies the path of the Java Runtime Environment (JRE) at the remote server and is used if on the remote server a JRE is not included in the system path. The path of the specified JRE is added to the start script at the remote server (sosftp.cmd respectively sosftp.sh).
	*
	*/
	@JSOptionDefinition(name = "classpath_base", description = "The parameter is used during installation of this", key = "classpath_base", type = "SOSOptionFolderName", mandatory = false)
	public SOSOptionFolderName	classpath_base				= new SOSOptionFolderName(this, conClassName + ".classpath_base", // HashMap-Key
																	"The parameter is used during installation of this", // Titel
																	"", // InitValue
																	"", // DefaultValue
																	false // isMandatory
															);

	/**
	 * \brief getclasspath_base
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionFolderName getclasspath_base() {
		return classpath_base;
	}

	/**
	 * \brief setclasspath_base
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_classpath_base
	 */
	public void setclasspath_base(SOSOptionFolderName p_classpath_base) {
		this.classpath_base = p_classpath_base;
	}

	/**
	* \var compress_files : This parameter specifies whether the content of the source files
	*
	This parameter specifies whether the content of the source files should be compressed by using a zip-algorithm or not. In case of sending files the files to be sent will be compressed.
	*
	*/
	@JSOptionDefinition(name = "compress_files", description = "This parameter specifies whether the content of the source files", key = "compress_files", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	compress_files	= new SOSOptionBoolean(this, conClassName + ".compress_files", // HashMap-Key
													"This parameter specifies whether the content of the source files", // Titel
													"false", // InitValue
													"false", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getcompress_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getcompress_files() {
		return compress_files;
	}

	/**
	 * \brief setcompress_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_compress_files
	 */
	public void setcompress_files(SOSOptionBoolean p_compress_files) {
		this.compress_files = p_compress_files;
	}

	/**
	* \var compressed_file_extension : Additional file-name extension for compressed files This parameter spe
	*
	This parameter specifies the file extension should target file compression be specified using the compress_files parameter.
	*
	*/
	@JSOptionDefinition(name = "compressed_file_extension", description = "Additional file-name extension for compressed files This parameter spe", key = "compressed_file_extension", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	compressed_file_extension	= new SOSOptionString(this, conClassName + ".compressed_file_extension", // HashMap-Key
																"Additional file-name extension for compressed files This parameter spe", // Titel
																".gz", // InitValue
																".gz", // DefaultValue
																false // isMandatory
														);

	/**
	 * \brief getcompressed_file_extension
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getcompressed_file_extension() {
		return compressed_file_extension;
	}

	/**
	 * \brief setcompressed_file_extension
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_compressed_file_extension
	 */
	public void setcompressed_file_extension(SOSOptionString p_compressed_file_extension) {
		this.compressed_file_extension = p_compressed_file_extension;
	}

	/**
	* \var current_pid : This parameter is used for Unix systems and - as o
	*
	This parameter is used for Unix systems and - as opposed to other parameters - is usually specified in the start script sosftp.sh. The value of the environment variable $$ is assigned, that contains the current process id (PID). The process id is used when writing an entry to a history file for each transfer (see parameter history).
	*
	*/
	@JSOptionDefinition(name = "current_pid", description = "This parameter is used for Unix systems and - as o", key = "current_pid", type = "SOSOptionProcessID", mandatory = false)
	public SOSOptionProcessID	current_pid	= new SOSOptionProcessID(this, conClassName + ".current_pid", // HashMap-Key
													"This parameter is used for Unix systems and - as o", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getcurrent_pid
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionProcessID getcurrent_pid() {
		return current_pid;
	}

	/**
	 * \brief setcurrent_pid
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_current_pid
	 */
	public void setcurrent_pid(SOSOptionProcessID p_current_pid) {
		this.current_pid = p_current_pid;
	}

	/**
	* \var file_path : This parameter is used alternatively to the parame
	*
	This parameter is used alternatively to the parameter file_spec to specify a single file for transfer. When receiving files the following applies: This parameter accepts the absolute name and path of file at the FTP/SFTP server that should be transferred. The file name has to include both name and path of the file at the FTP/SFTP server. The file will be stored unter its name in the directory that is specified by the parameter local_dir. The following parameters are ignored should this parameter be used: file_spec and remote_dir. When sending files the following applies: This parameter accepts the absolute name and path of file that should be transferred. An absolute path has to be specified. The file will be stored under its name in the directory at the FTP/SFTP server that has been specified by the parameter remote_dir. The following parameters are ignored should this parameter be used: file_spec and local_dir.
	*
	*/
	@JSOptionDefinition(name = "file_path", description = "This parameter is used alternatively to the parame", key = "file_path", type = "SOSOptionFileName", mandatory = false)
	public SOSOptionFileName	file_path	= new SOSOptionFileName(this, conClassName + ".file_path", // HashMap-Key
													"This parameter is used alternatively to the parame", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getfile_path
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionFileName getfile_path() {
		return file_path;
	}

	/**
	 * \brief setfile_path
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_file_path
	 */
	public void setfile_path(SOSOptionFileName p_file_path) {
		this.file_path = p_file_path;
	}

	/**
	* \var file_spec : file_spec This parameter expects a regular expressi
	*
	This parameter expects a regular expression to select files from a local directory or from an FTP/SFTP server (depending on the operation parameter values send or receive) to be transferred. For the operations send and receive either this parameter has to be specified or the parameter file_path or a list of file names as additional parameters.
	*
	*/
	@JSOptionDefinition(name = "file_spec", description = "file_spec This parameter expects a regular expressi", key = "file_spec", type = "SOSOptionRegExp", mandatory = false)
	public SOSOptionRegExp	file_spec	= new SOSOptionRegExp(this, conClassName + ".file_spec", // HashMap-Key
												"file_spec This parameter expects a regular expressi", // Titel
												"", // InitValue
												"", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getfile_spec
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionRegExp getfile_spec() {
		return file_spec;
	}

	/**
	 * \brief setfile_spec
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_file_spec
	 */
	public void setfile_spec(SOSOptionRegExp p_file_spec) {
		this.file_spec = p_file_spec;
	}

	public SOSOptionRegExp	FileNamePatternRegExp	= (SOSOptionRegExp) file_spec.SetAlias(conClassName + ".FileNamePatternRegExp");

	/**
	* \var file_spec2 : In addition to what is stated for the parameter fi
	*
	In addition to what is stated for the parameter file_spec additional parameters can be specified for up to 9 file sets like this: -file_spec=.*\.gif$ -local_dir=/tmp/1 -remote_dir=/tmp/1 -file_spec2=.*\.exe$::param_set_2 -param_set_2="transfer_mode=binary::remote_dir=/tmp/2::local_dir=/tmp/2" Within the file_spec2 parameter value the regular expression is separated by :: from the name of a file set. This name can freely be chosen, it consists of the characters 0-9, a-z and _. The name of the file set is used as a separate parameter in the command line. This parameter is assigend the list of parameters that should be valid for the specific file set. Therefore the names and values of individual parameters are specified in the form name=value::name2=value2 .... Such parameters are exclusively valid for the specific file set. The above sample causes all files with the extension .gif to be transferred from the local directory /tmp/1 to a directory with the same name on the target host. For files with the extension .exe a file set param_set_2 is specified that contains parameters that are specific for this file set, as binary transfer and different source and target directories. Please, consider that parameter file sets cannot specify parameters that control the connection to a target host, i.e. all files are transferred between the same local and remote hosts. However, the transfer direction can be changed, e.g. by specifying a different operation parameter for a file set.
	*
	*/
	@JSOptionDefinition(name = "file_spec2", description = "In addition to what is stated for the parameter fi", key = "file_spec2", type = "SOSOptionRegExp", mandatory = false)
	public SOSOptionRegExp	file_spec2				= new SOSOptionRegExp(this, conClassName + ".file_spec2", // HashMap-Key
															"In addition to what is stated for the parameter fi", // Titel
															" ", // InitValue
															" ", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief getfile_spec2
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionRegExp getfile_spec2() {
		return file_spec2;
	}

	/**
	 * \brief setfile_spec2
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_file_spec2
	 */
	public void setfile_spec2(SOSOptionRegExp p_file_spec2) {
		this.file_spec2 = p_file_spec2;
	}

	/**
	* \var force_files : This parameter specifies whether an error should b
	*
	This parameter specifies whether an error should be raised if no files could be found for transfer. The number of files to be transferred is determined by the file_spec or file_path parameters and can be restricted by the overwrite_files parameter should this be specified with the value false.
	*
	*/
	@JSOptionDefinition(name = "force_files", description = "This parameter specifies whether an error should b", key = "force_files", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	force_files	= new SOSOptionBoolean(this, conClassName + ".force_files", // HashMap-Key
												"This parameter specifies whether an error should b", // Titel
												"true", // InitValue
												"true", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getforce_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getforce_files() {
		return force_files;
	}

	/**
	 * \brief setforce_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_force_files
	 */
	public void setforce_files(SOSOptionBoolean p_force_files) {
		this.force_files = p_force_files;
	}

	/**
	* \var history : This parameter causes a history file to be written
	*
	This parameter causes a history file to be written in CSV format. The path and name of the history file is specified as value for this parameter. A history record is created for each file that has been transferred. A history file contains the following columns: guid A unique identifier for the history entry. This identifier is used for checking of duplicate entries in combination with Job Scheduler for Managed File Transfer. mandator A character that denominates the mandator of a file transfer, see respective parameter. transfer_timestamp The point in time when the transfer took place. pid The process id of the current process that executes the file transfer, see parameter current_pid. ppid The process id of the parent of the process that executes the file transfer, see respective parameter. operation One of the operations send or receive, see respective parameter. localhost The name of the host on which this program is executed. localhost_ip The IP address of the host on which this program is executed. local_user The name of the user account for which this program is executed. remote_host The name of the host to/from which a transfer is executed, see parameter host. remote_host_ip The IP address of the host to/from which a transfer is executed, see parameter host. remote_user The name of the user account for the remote host, see parameter user. protocol The protocol can be either ftp, sftp or ftps, see respective parameter. port The port on the remote host, see respective parameter. local_dir The local directory to/from which a file has been transferred, see respective parameter. remote_dir The remote directory to/from which a file has been transferred, see respective parameter. local_filename For send operations this is the original file name on the local host. For receive operations this is the resulting file name on the local host optionally having applied replacement operations, see parameter replacing. remote_filename For send operations this is the resulting file name on the remote host optionally having applied replacement operations, see parameter replacing. For receive operations this is the original file name on the remote host. file_size The size of the transferred file in bytes. md5 The value of the MD5 hash that is created from the file that was transferred. status The status can be either success or error. last_error_message Should an error have occurred then the last error message will be given in this column. log_filename The name of the log file, see respective parameter.
	*
	*/
	@JSOptionDefinition(name = "history", description = "This parameter causes a history file to be written", key = "history", type = "SOSOptionOutFileName", mandatory = false)
	public SOSOptionOutFileName	history	= new SOSOptionOutFileName(this, conClassName + ".history", // HashMap-Key
												"This parameter causes a history file to be written", // Titel
												" ", // InitValue
												" ", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief gethistory
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionOutFileName gethistory() {
		return history;
	}

	/**
	 * \brief sethistory
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_history
	 */
	public void sethistory(SOSOptionOutFileName p_history) {
		this.history = p_history;
	}

	public SOSOptionOutFileName	SOSFtpHistoryFileName	= (SOSOptionOutFileName) history.SetAlias(conClassName + ".SOSFtpHistoryFileName");

	/**
	* \var history_repeat : The parameter is used in order to synchronize para
	*
	The parameter is used in order to synchronize parallel write access to the history file by multiple instances of this program. This parameter specifies the maximum number of repeat intervals when trying to write to the history file if the history file is locked due to parallel instances of this program.
	*
	*/
	@JSOptionDefinition(name = "history_repeat", description = "The parameter is used in order to synchronize para", key = "history_repeat", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger		history_repeat			= new SOSOptionInteger(this, conClassName + ".history_repeat", // HashMap-Key
																"The parameter is used in order to synchronize para", // Titel
																"3", // InitValue
																"3", // DefaultValue
																false // isMandatory
														);

	/**
	 * \brief gethistory_repeat
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger gethistory_repeat() {
		return history_repeat;
	}

	/**
	 * \brief sethistory_repeat
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_history_repeat
	 */
	public void sethistory_repeat(SOSOptionInteger p_history_repeat) {
		this.history_repeat = p_history_repeat;
	}

	/**
	* \var history_repeat_interval : The parameter is used in order to synchronize para
	*
	The parameter is used in order to synchronize parallel write access to the history file by multiple instances of this program. This parameter specifies the the interval in seconds of repeated trials to write to the history file if the history file is locked due to parallel instances of this program.
	*
	*/
	@JSOptionDefinition(name = "history_repeat_interval", description = "The parameter is used in order to synchronize para", key = "history_repeat_interval", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger	history_repeat_interval	= new SOSOptionInteger(this, conClassName + ".history_repeat_interval", // HashMap-Key
															"The parameter is used in order to synchronize para", // Titel
															"1", // InitValue
															"1", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief gethistory_repeat_interval
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger gethistory_repeat_interval() {
		return history_repeat_interval;
	}

	/**
	 * \brief sethistory_repeat_interval
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_history_repeat_interval
	 */
	public void sethistory_repeat_interval(SOSOptionInteger p_history_repeat_interval) {
		this.history_repeat_interval = p_history_repeat_interval;
	}

	/**
	* \var host : Host-Name This parameter specifies th
	*
	This parameter specifies the hostname or IP address of the server to which a connection has to be made.
	*
	*/
	@JSOptionDefinition(name = "host", description = "Host-Name This parameter specifies th", key = "host", type = "SOSOptionHostName", mandatory = false)
	public SOSOptionHostName	host	= new SOSOptionHostName(this, conClassName + ".host", // HashMap-Key
												"Host-Name This parameter specifies th", // Titel
												" ", // InitValue
												" ", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief gethost
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionHostName gethost() {
		return host;
	}

	/**
	 * \brief sethost
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_host
	 */
	public void sethost(SOSOptionHostName p_host) {
		this.host = p_host;
	}

	public SOSOptionHostName	HostName		= (SOSOptionHostName) host.SetAlias(conClassName + ".HostName");

	/**
	* \var http_proxy_host : The value of this parameter is the host name or th
	*
	The value of this parameter is the host name or the IP address of a proxy used in order to establish a connection to the SSH server via SSL/TLS. The use of a proxy is optional and exclusively considered if the parameter protocol=ftps is used.
	*
	*/
	@JSOptionDefinition(name = "http_proxy_host", description = "The value of this parameter is the host name or th", key = "http_proxy_host", type = "SOSOptionString", mandatory = false)
	public SOSOptionString		http_proxy_host	= new SOSOptionString(this, conClassName + ".http_proxy_host", // HashMap-Key
														"The value of this parameter is the host name or th", // Titel
														" ", // InitValue
														" ", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief gethttp_proxy_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString gethttp_proxy_host() {
		return http_proxy_host;
	}

	/**
	 * \brief sethttp_proxy_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_http_proxy_host
	 */
	public void sethttp_proxy_host(SOSOptionString p_http_proxy_host) {
		this.http_proxy_host = p_http_proxy_host;
	}

	/**
	* \var http_proxy_port : This parameter specifies the port of a proxy that
	*
	This parameter specifies the port of a proxy that is used in order to establish a connection to the SSH server via SSL/TLS, see parameter http_proxy_host.
	*
	*/
	@JSOptionDefinition(name = "http_proxy_port", description = "This parameter specifies the port of a proxy that", key = "http_proxy_port", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	http_proxy_port	= new SOSOptionString(this, conClassName + ".http_proxy_port", // HashMap-Key
													"This parameter specifies the port of a proxy that", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief gethttp_proxy_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString gethttp_proxy_port() {
		return http_proxy_port;
	}

	/**
	 * \brief sethttp_proxy_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_http_proxy_port
	 */
	public void sethttp_proxy_port(SOSOptionString p_http_proxy_port) {
		this.http_proxy_port = p_http_proxy_port;
	}

	/**
	* \var jump_command : This parameter specifies a command that is to be e
	*
	This parameter specifies a command that is to be executed on the SSH server. Multiple commands can be separated by the command delimiter that is specified using the jump_command_delimiter parameter.
	*
	*/
	@JSOptionDefinition(name = "jump_command", description = "This parameter specifies a command that is to be e", key = "jump_command", type = "SOSOptionCommandString", mandatory = false)
	public SOSOptionCommandString	jump_command	= new SOSOptionCommandString(this, conClassName + ".jump_command", // HashMap-Key
															"This parameter specifies a command that is to be e", // Titel
															" ", // InitValue
															" ", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief getjump_command
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionCommandString getjump_command() {
		return jump_command;
	}

	/**
	 * \brief setjump_command
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_command
	 */
	public void setjump_command(SOSOptionCommandString p_jump_command) {
		this.jump_command = p_jump_command;
	}

	/**
	* \var jump_command_delimiter : Command delimiter characters are specified using t
	*
	Command delimiter characters are specified using this parameter. These delimiters can then be used in the jump_command parameter to seperate multiple commands. These commands are then excecuted in separate SSH sessions.
	*
	*/
	@JSOptionDefinition(name = "jump_command_delimiter", description = "Command delimiter characters are specified using t", key = "jump_command_delimiter", type = "SOSOptionString", mandatory = true)
	public SOSOptionString	jump_command_delimiter	= new SOSOptionString(this, conClassName + ".jump_command_delimiter", // HashMap-Key
															"Command delimiter characters are specified using t", // Titel
															"%%", // InitValue
															"%%", // DefaultValue
															true // isMandatory
													);

	/**
	 * \brief getjump_command_delimiter
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_command_delimiter() {
		return jump_command_delimiter;
	}

	/**
	 * \brief setjump_command_delimiter
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_command_delimiter
	 */
	public void setjump_command_delimiter(SOSOptionString p_jump_command_delimiter) {
		this.jump_command_delimiter = p_jump_command_delimiter;
	}

	/**
	* \var jump_command_script : This parameter can be used as an alternative to ju
	*
	This parameter can be used as an alternative to jump_command, jump_command_delimiter and jump_command_script_file. It contains script code which will be transferred to the remote host as a file and will then be executed there.
	*
	*/
	@JSOptionDefinition(name = "jump_command_script", description = "This parameter can be used as an alternative to ju", key = "jump_command_script", type = "SOSOptionCommandScript", mandatory = false)
	public SOSOptionCommandScript	jump_command_script	= new SOSOptionCommandScript(this, conClassName + ".jump_command_script", // HashMap-Key
																"This parameter can be used as an alternative to ju", // Titel
																" ", // InitValue
																" ", // DefaultValue
																false // isMandatory
														);

	/**
	 * \brief getjump_command_script
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionCommandScript getjump_command_script() {
		return jump_command_script;
	}

	/**
	 * \brief setjump_command_script
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_command_script
	 */
	public void setjump_command_script(SOSOptionCommandScript p_jump_command_script) {
		this.jump_command_script = p_jump_command_script;
	}

	/**
	* \var jump_command_script_file : This parameter can be used as an alternative to ju
	*
	This parameter can be used as an alternative to jump_command, jump_command_delimiter and jump_command_script. It contains the name of a script file, which will be transferred to the remote host and will then be executed there.
	*
	*/
	@JSOptionDefinition(name = "jump_command_script_file", description = "This parameter can be used as an alternative to ju", key = "jump_command_script_file", type = "SOSOptionCommandScriptFile", mandatory = false)
	public SOSOptionCommandScriptFile	jump_command_script_file	= new SOSOptionCommandScriptFile(this, conClassName + ".jump_command_script_file", // HashMap-Key
																			"This parameter can be used as an alternative to ju", // Titel
																			" ", // InitValue
																			" ", // DefaultValue
																			false // isMandatory
																	);

	/**
	 * \brief getjump_command_script_file
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionCommandScriptFile getjump_command_script_file() {
		return jump_command_script_file;
	}

	/**
	 * \brief setjump_command_script_file
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_command_script_file
	 */
	public void setjump_command_script_file(SOSOptionCommandScriptFile p_jump_command_script_file) {
		this.jump_command_script_file = p_jump_command_script_file;
	}

	/**
	* \var jump_host : When using a jump_host then files are first transf
	*
	When using a jump_host then files are first transferred to this host and then to the target system. Different protocols (FTP/SFTP) can be used for these transfer operations. Host or IP address of the jump_host from which or to which files should be transferred in a first operation.
	*
	*/
	@JSOptionDefinition(name = "jump_host", description = "When using a jump_host then files are first transf", key = "jump_host", type = "SOSOptionString", mandatory = false)
	public SOSOptionHostName	jump_host	= new SOSOptionHostName(this, conClassName + ".jump_host", // HashMap-Key
													"When using a jump_host then files are first transf", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getjump_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionHostName getjump_host() {
		return jump_host;
	}

	/**
	 * \brief setjump_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_host
	 */
	public void setjump_host(SOSOptionHostName p_jump_host) {
		this.jump_host = p_jump_host;
	}

	/**
	* \var jump_ignore_error : Should the value true be specified, then execution
	*
	Should the value true be specified, then execution errors caused by commands on the SSH server are ignored. Otherwise such execution errors will be reported.
	*
	*/
	@JSOptionDefinition(name = "jump_ignore_error", description = "Should the value true be specified, then execution", key = "jump_ignore_error", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	jump_ignore_error	= new SOSOptionBoolean(this, conClassName + ".jump_ignore_error", // HashMap-Key
														"Should the value true be specified, then execution", // Titel
														"false", // InitValue
														"false", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getjump_ignore_error
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getjump_ignore_error() {
		return jump_ignore_error;
	}

	/**
	 * \brief setjump_ignore_error
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_ignore_error
	 */
	public void setjump_ignore_error(SOSOptionBoolean p_jump_ignore_error) {
		this.jump_ignore_error = p_jump_ignore_error;
	}

	/**
	* \var jump_ignore_signal : Should the value true be specified, t
	*
	Should the value true be specified, then on Unix systems all signals will be ignored that terminate the execution of a command on the SSH server - if for example a command is terminated using kill. Note that by default errors will be reported for commands that are terminated by signals.
	*
	*/
	@JSOptionDefinition(name = "jump_ignore_signal", description = "Should the value true be specified, t", key = "jump_ignore_signal", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	jump_ignore_signal	= new SOSOptionBoolean(this, conClassName + ".jump_ignore_signal", // HashMap-Key
														"Should the value true be specified, t", // Titel
														"false", // InitValue
														"false", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getjump_ignore_signal
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getjump_ignore_signal() {
		return jump_ignore_signal;
	}

	/**
	 * \brief setjump_ignore_signal
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_ignore_signal
	 */
	public void setjump_ignore_signal(SOSOptionBoolean p_jump_ignore_signal) {
		this.jump_ignore_signal = p_jump_ignore_signal;
	}

	/**
	* \var jump_ignore_stderr : This job checks if any output to stderr has been c
	*
	This job checks if any output to stderr has been created by a command that is being executed on the SSH server and reports this as an error. Should the value true be specified for this parameter, then output to stderr will not be reported as an error by the Job Scheduler.
	*
	*/
	@JSOptionDefinition(name = "jump_ignore_stderr", description = "This job checks if any output to stderr has been c", key = "jump_ignore_stderr", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	jump_ignore_stderr	= new SOSOptionBoolean(this, conClassName + ".jump_ignore_stderr", // HashMap-Key
														"This job checks if any output to stderr has been c", // Titel
														"false", // InitValue
														"false", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getjump_ignore_stderr
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getjump_ignore_stderr() {
		return jump_ignore_stderr;
	}

	/**
	 * \brief setjump_ignore_stderr
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_ignore_stderr
	 */
	public void setjump_ignore_stderr(SOSOptionBoolean p_jump_ignore_stderr) {
		this.jump_ignore_stderr = p_jump_ignore_stderr;
	}

	/**
	* \var jump_password : Password for authentication with the jump_host.
	*
	Password for authentication with the jump_host.
	*
	*/
	@JSOptionDefinition(name = "jump_password", description = "Password for authentication with the jump_host.", key = "jump_password", type = "SOSOptionString", mandatory = false)
	public SOSOptionPassword	jump_password	= new SOSOptionPassword(this, conClassName + ".jump_password", // HashMap-Key
														"Password for authentication with the jump_host.", // Titel
														" ", // InitValue
														" ", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getjump_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionPassword getjump_password() {
		return jump_password;
	}

	/**
	 * \brief setjump_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_password
	 */
	public void setjump_password(SOSOptionPassword p_jump_password) {
		this.jump_password = p_jump_password;
	}

	/**
	* \var jump_port : Port on the jump_host by which files should be tra
	*
	Port on the jump_host by which files should be transferred. For FTP this is usually port 21, for SFTP this is usually port 22.
	*
	*/
	@JSOptionDefinition(name = "jump_port", description = "Port on the jump_host by which files should be tra", key = "jump_port", type = "SOSOptionString", mandatory = false)
	public SOSOptionPortNumber	jump_port	= new SOSOptionPortNumber(this, conClassName + ".jump_port", // HashMap-Key
													"Port on the jump_host by which files should be tra", // Titel
													"22", // InitValue
													"22", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getjump_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionPortNumber getjump_port() {
		return jump_port;
	}

	/**
	 * \brief setjump_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_port
	 */
	public void setjump_port(SOSOptionPortNumber p_jump_port) {
		this.jump_port = p_jump_port;
	}

	/**
	* \var jump_protocol : When using a jump_host then files are first transf
	*
	When using a jump_host then files are first transferred to this host and then to the target system. Different protocols (FTP/SFTP) can be used for these transfer operations. This parameter expects ftp, sftp or ftps to be specified. If sftp is used, then the jump_ssh_* parameters will be considered.
	*
	*/
	@JSOptionDefinition(name = "jump_protocol", description = "When using a jump_host then files are first transf", key = "jump_protocol", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	jump_protocol	= new SOSOptionString(this, conClassName + ".jump_protocol", // HashMap-Key
													"When using a jump_host then files are first transf", // Titel
													"sftp", // InitValue
													"sftp", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getjump_protocol
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_protocol() {
		return jump_protocol;
	}

	/**
	 * \brief setjump_protocol
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_protocol
	 */
	public void setjump_protocol(SOSOptionString p_jump_protocol) {
		this.jump_protocol = p_jump_protocol;
	}

	/**
	* \var jump_proxy_host : The value of this parameter is the host name or th
	*
	The value of this parameter is the host name or the IP address of a proxy used in order to establish a connection to the jump host. The use of a proxy is optional.
	*
	*/
	@JSOptionDefinition(name = "jump_proxy_host", description = "The value of this parameter is the host name or th", key = "jump_proxy_host", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	jump_proxy_host	= new SOSOptionString(this, conClassName + ".jump_proxy_host", // HashMap-Key
													"The value of this parameter is the host name or th", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getjump_proxy_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_proxy_host() {
		return jump_proxy_host;
	}

	/**
	 * \brief setjump_proxy_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_proxy_host
	 */
	public void setjump_proxy_host(SOSOptionString p_jump_proxy_host) {
		this.jump_proxy_host = p_jump_proxy_host;
	}

	/**
	* \var jump_proxy_password : This parameter specifies the password for the prox
	*
	This parameter specifies the password for the proxy server user account, should a proxy be used in order to connect to the jump host, see parameter jump_proxy_host.
	*
	*/
	@JSOptionDefinition(name = "jump_proxy_password", description = "This parameter specifies the password for the prox", key = "jump_proxy_password", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	jump_proxy_password	= new SOSOptionString(this, conClassName + ".jump_proxy_password", // HashMap-Key
														"This parameter specifies the password for the prox", // Titel
														" ", // InitValue
														" ", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getjump_proxy_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_proxy_password() {
		return jump_proxy_password;
	}

	/**
	 * \brief setjump_proxy_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_proxy_password
	 */
	public void setjump_proxy_password(SOSOptionString p_jump_proxy_password) {
		this.jump_proxy_password = p_jump_proxy_password;
	}

	/**
	* \var jump_proxy_port : This parameter specifies the port of a proxy that
	*
	This parameter specifies the port of a proxy that is used in order to establish a connection to the jump host, see parameter jump_proxy_host.
	*
	*/
	@JSOptionDefinition(name = "jump_proxy_port", description = "This parameter specifies the port of a proxy that", key = "jump_proxy_port", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	jump_proxy_port	= new SOSOptionString(this, conClassName + ".jump_proxy_port", // HashMap-Key
													"This parameter specifies the port of a proxy that", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getjump_proxy_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_proxy_port() {
		return jump_proxy_port;
	}

	/**
	 * \brief setjump_proxy_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_proxy_port
	 */
	public void setjump_proxy_port(SOSOptionString p_jump_proxy_port) {
		this.jump_proxy_port = p_jump_proxy_port;
	}

	/**
	* \var jump_proxy_user : The value of this parameter specifies the user acc
	*
	The value of this parameter specifies the user account for authentication by the proxy server should a proxy be used in order to connect to the jump host, see parameter jump_proxy_host.
	*
	*/
	@JSOptionDefinition(name = "jump_proxy_user", description = "The value of this parameter specifies the user acc", key = "jump_proxy_user", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	jump_proxy_user	= new SOSOptionString(this, conClassName + ".jump_proxy_user", // HashMap-Key
													"The value of this parameter specifies the user acc", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getjump_proxy_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_proxy_user() {
		return jump_proxy_user;
	}

	/**
	 * \brief setjump_proxy_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_proxy_user
	 */
	public void setjump_proxy_user(SOSOptionString p_jump_proxy_user) {
		this.jump_proxy_user = p_jump_proxy_user;
	}

	/**
	* \var jump_simulate_shell : Should the value true be specified for this parame
	*
	Should the value true be specified for this parameter, then a login to a shell is simulated to execute commands. Some scripts may cause problems if no shell is present.
	*
	*/
	@JSOptionDefinition(name = "jump_simulate_shell", description = "Should the value true be specified for this parame", key = "jump_simulate_shell", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	jump_simulate_shell	= new SOSOptionBoolean(this, conClassName + ".jump_simulate_shell", // HashMap-Key
														"Should the value true be specified for this parame", // Titel
														"false", // InitValue
														"false", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getjump_simulate_shell
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getjump_simulate_shell() {
		return jump_simulate_shell;
	}

	/**
	 * \brief setjump_simulate_shell
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_simulate_shell
	 */
	public void setjump_simulate_shell(SOSOptionBoolean p_jump_simulate_shell) {
		this.jump_simulate_shell = p_jump_simulate_shell;
	}

	/**
	* \var jump_simulate_shell_inactivity_timeout : If no new characters are written to stdout or stde
	*
	If no new characters are written to stdout or stderr after the given number of milliseconds, then it is assumed that the command has been carried out and that the shell is waiting for the next command.
	*
	*/
	@JSOptionDefinition(name = "jump_simulate_shell_inactivity_timeout", description = "If no new characters are written to stdout or stde", key = "jump_simulate_shell_inactivity_timeout", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger	jump_simulate_shell_inactivity_timeout	= new SOSOptionInteger(this, conClassName + ".jump_simulate_shell_inactivity_timeout", // HashMap-Key
																			"If no new characters are written to stdout or stde", // Titel
																			" ", // InitValue
																			" ", // DefaultValue
																			false // isMandatory
																	);

	/**
	 * \brief getjump_simulate_shell_inactivity_timeout
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger getjump_simulate_shell_inactivity_timeout() {
		return jump_simulate_shell_inactivity_timeout;
	}

	/**
	 * \brief setjump_simulate_shell_inactivity_timeout
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_simulate_shell_inactivity_timeout
	 */
	public void setjump_simulate_shell_inactivity_timeout(SOSOptionInteger p_jump_simulate_shell_inactivity_timeout) {
		this.jump_simulate_shell_inactivity_timeout = p_jump_simulate_shell_inactivity_timeout;
	}

	/**
	* \var jump_simulate_shell_login_timeout : If no new characters are written to stdout or stde
	*
	If no new characters are written to stdout or stderr after the given number of milliseconds, then it is assumed that the login has been carried out and that the shell is waiting for the next command.
	*
	*/
	@JSOptionDefinition(name = "jump_simulate_shell_login_timeout", description = "If no new characters are written to stdout or stde", key = "jump_simulate_shell_login_timeout", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger	jump_simulate_shell_login_timeout	= new SOSOptionInteger(this, conClassName + ".jump_simulate_shell_login_timeout", // HashMap-Key
																		"If no new characters are written to stdout or stde", // Titel
																		" ", // InitValue
																		" ", // DefaultValue
																		false // isMandatory
																);

	/**
	 * \brief getjump_simulate_shell_login_timeout
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger getjump_simulate_shell_login_timeout() {
		return jump_simulate_shell_login_timeout;
	}

	/**
	 * \brief setjump_simulate_shell_login_timeout
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_simulate_shell_login_timeout
	 */
	public void setjump_simulate_shell_login_timeout(SOSOptionInteger p_jump_simulate_shell_login_timeout) {
		this.jump_simulate_shell_login_timeout = p_jump_simulate_shell_login_timeout;
	}

	/**
	* \var jump_simulate_shell_prompt_trigger : The expected command line prompt. Using this promp
	*
	The expected command line prompt. Using this prompt the program tries to find out if commands may be entered or have been carried out. If no prompt can be configured, then timeout parameters have to be used in order to check if the shell is ready to accept commands.
	*
	*/
	@JSOptionDefinition(name = "jump_simulate_shell_prompt_trigger", description = "The expected command line prompt. Using this promp", key = "jump_simulate_shell_prompt_trigger", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	jump_simulate_shell_prompt_trigger	= new SOSOptionString(this, conClassName + ".jump_simulate_shell_prompt_trigger", // HashMap-Key
																		"The expected command line prompt. Using this promp", // Titel
																		" ", // InitValue
																		" ", // DefaultValue
																		false // isMandatory
																);

	/**
	 * \brief getjump_simulate_shell_prompt_trigger
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_simulate_shell_prompt_trigger() {
		return jump_simulate_shell_prompt_trigger;
	}

	/**
	 * \brief setjump_simulate_shell_prompt_trigger
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_simulate_shell_prompt_trigger
	 */
	public void setjump_simulate_shell_prompt_trigger(SOSOptionString p_jump_simulate_shell_prompt_trigger) {
		this.jump_simulate_shell_prompt_trigger = p_jump_simulate_shell_prompt_trigger;
	}

	/**
	* \var jump_ssh_auth_file : This parameter specifies the path and name of a us
	*
	This parameter specifies the path and name of a user's private key file used for login to the SSH server of the jump_host. This parameter must be specified if the publickey authentication method has been specified in the jump_ssh_auth_method parameter. Should the private key file be secured by a password, then this password has to be specified using the jump_password parameter.
	*
	*/
	@JSOptionDefinition(name = "jump_ssh_auth_file", description = "This parameter specifies the path and name of a us", key = "jump_ssh_auth_file", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	jump_ssh_auth_file	= new SOSOptionString(this, conClassName + ".jump_ssh_auth_file", // HashMap-Key
														"This parameter specifies the path and name of a us", // Titel
														" ", // InitValue
														" ", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getjump_ssh_auth_file
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_ssh_auth_file() {
		return jump_ssh_auth_file;
	}

	/**
	 * \brief setjump_ssh_auth_file
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_ssh_auth_file
	 */
	public void setjump_ssh_auth_file(SOSOptionString p_jump_ssh_auth_file) {
		this.jump_ssh_auth_file = p_jump_ssh_auth_file;
	}

	/**
	* \var jump_ssh_auth_method : This parameter specifies the authentication method
	*
	This parameter specifies the authentication method for the SSH server - the publickey and password methods are supported. When the publickey authentication method is used, then the path name of the private key file must be set in the jump_ssh_auth_file parameter. Should the private key file be secured by a passphrase then this passphrase has to be specified by the jump_password parameter. For the password authentication method the password for the user account has to be specified using the jump_password parameter. The authentication methods that are enabled depend on the SSH server configuration. Not all SSH servers are configured for password authentication.
	*
	*/
	@JSOptionDefinition(name = "jump_ssh_auth_method", description = "This parameter specifies the authentication method", key = "jump_ssh_auth_method", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	jump_ssh_auth_method	= new SOSOptionString(this, conClassName + ".jump_ssh_auth_method", // HashMap-Key
															"This parameter specifies the authentication method", // Titel
															" ", // InitValue
															" ", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief getjump_ssh_auth_method
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_ssh_auth_method() {
		return jump_ssh_auth_method;
	}

	/**
	 * \brief setjump_ssh_auth_method
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_ssh_auth_method
	 */
	public void setjump_ssh_auth_method(SOSOptionString p_jump_ssh_auth_method) {
		this.jump_ssh_auth_method = p_jump_ssh_auth_method;
	}

	/**
	* \var jump_user : User name for authentication with the jump_host.
	*
	User name for authentication with the jump_host.
	*
	*/
	@JSOptionDefinition(name = "jump_user", description = "User name for authentication with the jump_host.", key = "jump_user", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	jump_user	= new SOSOptionString(this, conClassName + ".jump_user", // HashMap-Key
												"User name for authentication with the jump_host.", // Titel
												" ", // InitValue
												" ", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getjump_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getjump_user() {
		return jump_user;
	}

	/**
	 * \brief setjump_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_jump_user
	 */
	public void setjump_user(SOSOptionString p_jump_user) {
		this.jump_user = p_jump_user;
	}

	/**
	* \var local_dir : local_dir Local directory into which or from which
	*
	Local directory into which or from which files should be transferred. By default the current working directory is used. Besides paths in the local file system UNC path names are supported that could be used to address remote server systems: \\somehost\somedirectory can be used in the same way as //somehost/somedirectory to transfer files from an FTP/SFTP server to a different remote server system. Moreover, you could specify URIs for a file schema as in file:////somehost/somedirectory. Please, consider the required number of slashes. file URIs are subject to the following limitations due to constraints of the underlying Java JRE: File names and path names must not contain any spaces. Authentication by authority strings as in file:////user:password@somehost/somedirectory is not supported.
	*
	*/
	@JSOptionDefinition(name = "local_dir", description = "local_dir Local directory into which or from which", key = "local_dir", type = "SOSOptionFolderName", mandatory = true)
	public SOSOptionFolderName	local_dir	= new SOSOptionFolderName(this, conClassName + ".local_dir", // HashMap-Key
													"local_dir Local directory into which or from which", // Titel
													"", // InitValue
													"", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getlocal_dir
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionFolderName getlocal_dir() {
		return local_dir;
	}

	/**
	 * \brief setlocal_dir
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_local_dir
	 */
	public void setlocal_dir(SOSOptionFolderName p_local_dir) {
		this.local_dir = p_local_dir;
	}

	/**
	* \var mandator : This parameter specifies the mandator for which a
	*
	This parameter specifies the mandator for which a file transfer is effected. The mandator is added to an optional history file, see parameter history and has no technical relevance for the transfer.
	*
	*/
	@JSOptionDefinition(name = "mandator", description = "This parameter specifies the mandator for which a", key = "mandator", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	mandator	= new SOSOptionString(this, conClassName + ".mandator", // HashMap-Key
												"This parameter specifies the mandator for which a", // Titel
												"SOS", // InitValue
												"SOS", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getmandator
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getmandator() {
		return mandator;
	}

	/**
	 * \brief setmandator
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_mandator
	 */
	public void setmandator(SOSOptionString p_mandator) {
		this.mandator = p_mandator;
	}

	/**
	* \var operation : Operation to be executed send, receive, remove,
	*
	send, receive, remove, execute or install. send - transfer files by FTP/SFTP to a remote server receive - transfer files by FTP/SFTP from a remote server remove - remove files by FTP/SFTP on a remote server execute - execute a command by SSH on a remote server install - install SOSFTP on a remote server
	*
	*/
	@JSOptionDefinition(name = "operation", description = "Operation to be executed send, receive, remove,", key = "operation", type = "SOSOptionStringValueList", mandatory = true)
	public SOSOptionJadeOperation	operation	= new SOSOptionJadeOperation(this, conClassName + ".operation", // HashMap-Key
														"Operation to be executed send, receive, remove,", // Titel
														"send", // InitValue
														"send", // DefaultValue
														true // isMandatory
												);

	/**
	 * \brief getoperation
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionJadeOperation getoperation() {
		return operation;
	}

	/**
	 * \brief setoperation
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_operation
	 */
	public void setoperation(SOSOptionJadeOperation p_operation) {
		this.operation = p_operation;
	}

	/**
	* \var overwrite_files : This parameter specifies if existing files should
	*
	This parameter specifies if existing files should be overwritten. Should this parameter be used with force_files und should no files be transferred due to overwrite protection then an error will be raised stating that "no matching files" could be found.
	*
	*/
	@JSOptionDefinition(name = "overwrite_files", description = "This parameter specifies if existing files should", key = "overwrite_files", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	overwrite_files	= new SOSOptionBoolean(this, conClassName + ".overwrite_files", // HashMap-Key
													"This parameter specifies if existing files should", // Titel
													"true", // InitValue
													"true", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getoverwrite_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getoverwrite_files() {
		return overwrite_files;
	}

	/**
	 * \brief setoverwrite_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_overwrite_files
	 */
	public void setoverwrite_files(SOSOptionBoolean p_overwrite_files) {
		this.overwrite_files = p_overwrite_files;
	}

	/**
	* \var passive_mode : passive_mode Passive mode for FTP is often used wit
	*
	Passive mode for FTP is often used with firewalls. Valid values are 0 or 1.
	*
	*/
	@JSOptionDefinition(name = "passive_mode", description = "passive_mode Passive mode for FTP is often used wit", key = "passive_mode", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	passive_mode	= new SOSOptionBoolean(this, conClassName + ".passive_mode", // HashMap-Key
													"passive_mode Passive mode for FTP is often used wit", // Titel
													"false", // InitValue
													"false", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getpassive_mode
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getpassive_mode() {
		return passive_mode;
	}

	/**
	 * \brief setpassive_mode
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_passive_mode
	 */
	public void setpassive_mode(SOSOptionBoolean p_passive_mode) {
		this.passive_mode = p_passive_mode;
	}

	public SOSOptionBoolean		FTPTransferModeIsPassive	= (SOSOptionBoolean) passive_mode.SetAlias(conClassName + ".FTPTransferModeIsPassive");

	/**
	* \var password : Password for UserID Password for a
	*
	Password for authentication at the FTP/SFTP server. For SSH/SFTP connections that make use of public/private key authentication the password parameter is specified for the passphrase that optionally secures a private key.
	*
	*/
	@JSOptionDefinition(name = "password", description = "Password for UserID Password for a", key = "password", type = "SOSOptionPassword", mandatory = false)
	public SOSOptionPassword	password					= new SOSOptionPassword(this, conClassName + ".password", // HashMap-Key
																	"Password for UserID Password for a", // Titel
																	" ", // InitValue
																	" ", // DefaultValue
																	false // isMandatory
															);

	/**
	 * \brief getpassword
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionPassword getpassword() {
		return password;
	}

	/**
	 * \brief setpassword
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_password
	 */
	public void setpassword(SOSOptionPassword p_password) {
		this.password = p_password;
	}

	/**
	* \var poll_interval : This parameter specifies the interval in seconds
	*
	This parameter specifies the interval in seconds, how often a file is polled for within the duration that is specified by the parameter poll_timeout.
	*
	*/
	@JSOptionDefinition(name = "poll_interval", description = "This parameter specifies the interval in seconds", key = "poll_interval", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger	poll_interval	= new SOSOptionInteger(this, conClassName + ".poll_interval", // HashMap-Key
													"This parameter specifies the interval in seconds", // Titel
													"60", // InitValue
													"60", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getpoll_interval
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger getpoll_interval() {
		return poll_interval;
	}

	/**
	 * \brief setpoll_interval
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_poll_interval
	 */
	public void setpoll_interval(SOSOptionInteger p_poll_interval) {
		this.poll_interval = p_poll_interval;
	}

	/**
	* \var poll_minfiles : This parameter specifies the number of files tha
	*
	This parameter specifies the number of files that have to be found during the polling period in order to cause the transfer to start. This parameter is used exclusively with the parameters poll_timeout.
	*
	*/
	@JSOptionDefinition(name = "poll_minfiles", description = "This parameter specifies the number of files tha", key = "poll_minfiles", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger	poll_minfiles	= new SOSOptionInteger(this, conClassName + ".poll_minfiles", // HashMap-Key
													"This parameter specifies the number of files tha", // Titel
													"1", // InitValue
													"1", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getpoll_minfiles
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger getpoll_minfiles() {
		return poll_minfiles;
	}

	/**
	 * \brief setpoll_minfiles
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_poll_minfiles
	 */
	public void setpoll_minfiles(SOSOptionInteger p_poll_minfiles) {
		this.poll_minfiles = p_poll_minfiles;
	}

	/**
	* \var poll_timeout : This parameter specifies the time in minutes, how
	*
	This parameter specifies the time in minutes, how long a file is polled for. If a file becomes available within the time specified then it will be transferred, otherwise an error "no matching files" will be raised.
	*
	*/
	@JSOptionDefinition(name = "poll_timeout", description = "This parameter specifies the time in minutes, how", key = "poll_timeout", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger	poll_timeout	= new SOSOptionInteger(this, conClassName + ".poll_timeout", // HashMap-Key
													"This parameter specifies the time in minutes, how", // Titel
													"0", // InitValue
													"0", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getpoll_timeout
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger getpoll_timeout() {
		return poll_timeout;
	}

	/**
	 * \brief setpoll_timeout
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_poll_timeout
	 */
	public void setpoll_timeout(SOSOptionInteger p_poll_timeout) {
		this.poll_timeout = p_poll_timeout;
	}

	/**
	* \var port : Port-Number to be used for Data-Transfer
	*
	Port by which files should be transferred. For FTP this is usually port 21, for SFTP this is usually port 22.
	*
	*/
	@JSOptionDefinition(name = "port", description = "Port-Number to be used for Data-Transfer", key = "port", type = "SOSOptionPortNumber", mandatory = true)
	public SOSOptionPortNumber	port	= new SOSOptionPortNumber(this, conClassName + ".port", // HashMap-Key
												"Port-Number to be used for Data-Transfer", // Titel
												"21", // InitValue
												"21", // DefaultValue
												true // isMandatory
										);

	/**
	 * \brief getport
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionPortNumber getport() {
		return port;
	}

	/**
	 * \brief setport
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_port
	 */
	public void setport(SOSOptionPortNumber p_port) {
		this.port = p_port;
	}

	/**
	* \var ppid : This parameter is used for Unix systems and - as o
	*
	This parameter is used for Unix systems and - as opposed to other parameters - is usually specified in the start script sosftp.sh. The value of the environment variable $PPID is assigned, that contains the process id of the current parent process (PPID). The parent process id is used when writing an entry to a history file for each transfer (see parameter history).
	*
	*/
	@JSOptionDefinition(name = "ppid", description = "This parameter is used for Unix systems and - as o", key = "ppid", type = "SOSOptionProcessID", mandatory = false)
	public SOSOptionProcessID	ppid	= new SOSOptionProcessID(this, conClassName + ".ppid", // HashMap-Key
												"This parameter is used for Unix systems and - as o", // Titel
												" ", // InitValue
												" ", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getppid
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionProcessID getppid() {
		return ppid;
	}

	/**
	 * \brief setppid
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_ppid
	 */
	public void setppid(SOSOptionProcessID p_ppid) {
		this.ppid = p_ppid;
	}

	public SOSOptionProcessID	ParentProcessID	= (SOSOptionProcessID) ppid.SetAlias(conClassName + ".ParentProcessID");

	/**
	* \var profile : The Name of a Profile-Section to be executed
	*
	If a configuration file is being used (see parameter settings), then this parameter specifies a name of a section within the configuration file. Such sections, i.e. profiles, specify parameters as pairs of names and values that otherwise would be specified by command line parameters. At the command line the name of the configuration file and the profile are specified like this: sosftp.sh -operation=send -settings=settings.ini -profile=sample_transfer ...
	*
	*/
	@JSOptionDefinition(name = "profile", description = "The Name of a Profile-Section to be executed", key = "profile", type = "SOSOptionString", mandatory = false)
	public SOSOptionString		profile			= new SOSOptionString(this, conClassName + ".profile", // HashMap-Key
														"The Name of a Profile-Section to be executed", // Titel
														" ", // InitValue
														" ", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getprofile
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getprofile() {
		return profile;
	}

	/**
	 * \brief setprofile
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_profile
	 */
	public void setprofile(SOSOptionString p_profile) {
		this.profile = p_profile;
	}

	public SOSOptionString			SectionName	= (SOSOptionString) profile.SetAlias(conClassName + ".SectionName");

	/**
	* \var protocol : Type of requested Datatransfer The values ftp, sftp
	*
	The values ftp, sftp or ftps are valid for this parameter. If sftp is used, then the ssh_* parameters will be applied.
	*
	*/
	@JSOptionDefinition(name = "protocol", description = "Type of requested Datatransfer The values ftp, sftp", key = "protocol", type = "SOSOptionStringValueList", mandatory = true)
	public SOSOptionTransferType	protocol	= new SOSOptionTransferType(this, conClassName + ".protocol", // HashMap-Key
														"Type of requested Datatransfer The values ftp, sftp", // Titel
														"ftp", // InitValue
														"ftp", // DefaultValue
														true // isMandatory
												);

	/**
	 * \brief getprotocol
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionTransferType getprotocol() {
		return protocol;
	}

	/** 
	 * \brief setprotocol
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_protocol
	 */
	public void setprotocol(SOSOptionTransferType p_protocol) {
		this.protocol = p_protocol;
	}

	public SOSOptionTransferType	TransferProtocol	= (SOSOptionTransferType) protocol.SetAlias(conClassName + ".TransferProtocol");

	/**
	* \var recursive : This parameter specifies if files from subdirector
	*
	This parameter specifies if files from subdirectories should be transferred recursively. Recursive processing is specified by one of the values yes or true. Regular expression matches apply to files from subdirectories as specified by the parameter file_spec.
	*
	*/
	@JSOptionDefinition(name = "recursive", description = "This parameter specifies if files from subdirector", key = "recursive", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean			recursive			= new SOSOptionBoolean(this, conClassName + ".recursive", // HashMap-Key
																"This parameter specifies if files from subdirector", // Titel
																"false", // InitValue
																"false", // DefaultValue
																false // isMandatory
														);

	/**
	 * \brief getrecursive
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getrecursive() {
		return recursive;
	}

	/**
	 * \brief setrecursive
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_recursive
	 */
	public void setrecursive(SOSOptionBoolean p_recursive) {
		this.recursive = p_recursive;
	}

	public SOSOptionBoolean		RecurseSubFolders	= (SOSOptionBoolean) recursive.SetAlias(conClassName + ".RecurseSubFolders");

	/**
	* \var remote_dir : remote_dir Directory at the FTP/SFTP server from wh
	*
	Directory at the FTP/SFTP server from which or to which files should be transferred. By default the home directory of the user at the FTP/SFTP server is used.
	*
	*/
	@JSOptionDefinition(name = "remote_dir", description = "remote_dir Directory at the FTP/SFTP server from wh", key = "remote_dir", type = "SOSOptionFolderName", mandatory = true)
	public SOSOptionFolderName	remote_dir			= new SOSOptionFolderName(this, conClassName + ".remote_dir", // HashMap-Key
															"remote_dir Directory at the FTP/SFTP server from wh", // Titel
															".", // InitValue
															".", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief getremote_dir
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionFolderName getremote_dir() {
		return remote_dir;
	}

	/**
	 * \brief setremote_dir
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_remote_dir
	 */
	public void setremote_dir(SOSOptionFolderName p_remote_dir) {
		this.remote_dir = p_remote_dir;
	}

	/**
	* \var remove_files : This parameter specifies whether files on the FTP/
	*
	This parameter specifies whether files on the FTP/SFTP server should be removed after transfer.
	*
	*/
	@JSOptionDefinition(name = "remove_files", description = "This parameter specifies whether files on the FTP/", key = "remove_files", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	remove_files				= new SOSOptionBoolean(this, conClassName + ".remove_files", // HashMap-Key
																"This parameter specifies whether files on the FTP/", // Titel
																"false", // InitValue
																"false", // DefaultValue
																false // isMandatory
														);

	public SOSOptionBoolean	DeleteFilesAfterTransfer	= (SOSOptionBoolean) remove_files.SetAlias(conClassName + ".DeleteFilesAfterTransfer");

	/**
	 * \brief getremove_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getremove_files() {
		return remove_files;
	}

	/**
	 * \brief setremove_files
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_remove_files
	 */
	public void setremove_files(SOSOptionBoolean p_remove_files) {
		this.remove_files = p_remove_files;
	}

	/**
	* \var replacement : String for replacement of matching character seque
	*
	String for replacement of matching character sequences within file names that are specified with the value of the parameter replacing. If multiple "capturing groups" shall be replaced then one replacement string per group has to be specified. These strings are separated by a semicolon ";": replacement: aa;[filename:];bb Supports masks for substitution in the file name with format strings that are enclosed with [ and ] . The following format strings are supported: [date: date format ] date format must be a valid Java data format string, e.g. yyyyMMddHHmmss , yyyy-MM-dd.HHmmss etc. [filename:] Will be substituted by the original file name including the file extension. [filename:lowercase] Will be substituted by the original file name including the file extension with all characters converted to lower case. [filename:uppercase] Will be substituted by the original file name including the file extension with all characters converted to upper case. Requires the parameter replacing to be specified.
	*
	*/
	@JSOptionDefinition(name = "replacement", description = "String for replacement of matching character seque", key = "replacement", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	replacement	= new SOSOptionString(this, conClassName + ".replacement", // HashMap-Key
												"String for replacement of matching character seque", // Titel
												null, // InitValue
												null, // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getreplacement
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getreplacement() {
		return replacement;
	}

	/**
	 * \brief setreplacement
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_replacement
	 */
	public void setreplacement(SOSOptionString p_replacement) {
		this.replacement = p_replacement;
	}

	public SOSOptionString	ReplaceWith	= (SOSOptionString) replacement.SetAlias(conClassName + ".ReplaceWith");

	/**
	* \var replacing : Regular expression for filename replacement with
	*
	Regular expression for filename replacement with the value of the parameter replacement. If the expression matches the filename then the groups found are replaced. a) For replacement "capturing groups" are used. Only the content of the capturing groups is replaced. Replacements are separated by a semicolon ";". Example: replacing : (1)abc(12)def(.*) replacement : A;BB;CCC Input file: 1abc12def123.txt Output file: AabcBBdefCCC b) If no "capturing groups" are specified then the entire match is replaced. Example: replacing : Hello replacement : 1234 Input file: Hello_World.txt Output file: 1234_World.txt Requires the parameter replacement to be specified.
	*
	*/
	@JSOptionDefinition(name = "replacing", description = "Regular expression for filename replacement with", key = "replacing", type = "SOSOptionRegExp", mandatory = false)
	public SOSOptionRegExp	replacing	= new SOSOptionRegExp(this, conClassName + ".replacing", // HashMap-Key
												"Regular expression for filename replacement with", // Titel
												" ", // InitValue
												" ", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getreplacing
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionRegExp getreplacing() {
		return replacing;
	}

	/**
	 * \brief setreplacing
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_replacing
	 */
	public void setreplacing(SOSOptionRegExp p_replacing) {
		this.replacing = p_replacing;
	}

	public SOSOptionRegExp		ReplaceWhat	= (SOSOptionRegExp) replacing.SetAlias(conClassName + ".ReplaceWhat");

	/**
	* \var root : The parameter specifies the directory in which thi
	*
	The parameter specifies the directory in which this program is allowed to create temporary files. Temporary files are required if due to the parameter setting jump_host files have to be stored on an intermediary server and will be removed after completion of the transfer. Without this parameter the temporary directory is used as provided by the operating system.
	*
	*/
	@JSOptionDefinition(name = "root", description = "The parameter specifies the directory in which thi", key = "root", type = "SOSOptionFolderName", mandatory = false)
	public SOSOptionFolderName	root		= new SOSOptionFolderName(this, conClassName + ".root", // HashMap-Key
													"The parameter specifies the directory in which thi", // Titel
													"", // InitValue
													"", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getroot
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionFolderName getroot() {
		return root;
	}

	/**
	 * \brief setroot
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_root
	 */
	public void setroot(SOSOptionFolderName p_root) {
		this.root = p_root;
	}

	public SOSOptionFolderName	TempFolderName	= (SOSOptionFolderName) root.SetAlias(conClassName + ".TempFolderName");

	/**
	* \var scheduler_host : This parameter specifies the host name or IP addre
	*
	This parameter specifies the host name or IP address of a server for which Job Scheduler is operated for Managed File Transfer. The contents of an optional history file (see parameter history), is added to a central database by Job Scheduler. This parameter causes the transfer of the history entries for the current transfer by UDP to Job Scheduler. Should Job Scheduler not be accessible then no errors are reported, instead, the contents of the history will automaticall be processed later on.
	*
	*/
	@JSOptionDefinition(name = "scheduler_host", description = "This parameter specifies the host name or IP addre", key = "scheduler_host", type = "SOSOptionString", mandatory = false)
	public SOSOptionHostName		scheduler_host	= new SOSOptionHostName(this, conClassName + ".scheduler_host", // HashMap-Key
														"This parameter specifies the host name or IP addre", // Titel
														"", // InitValue
														"", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getscheduler_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionHostName getscheduler_host() {
		return scheduler_host;
	}

	/**
	 * \brief setscheduler_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_scheduler_host
	 */
	public void setscheduler_host(SOSOptionHostName p_scheduler_host) {
		this.scheduler_host = p_scheduler_host;
	}

	/**
	* \var scheduler_job_chain : The name of a job chain for Managed File Transfer
	*
	The name of a job chain for Managed File Transfer with Job Scheduler, see parameter scheduler_host. The job chain accepts history entries and performs an import into a central database.
	*
	*/
	@JSOptionDefinition(name = "scheduler_job_chain", description = "The name of a job chain for Managed File Transfer", key = "scheduler_job_chain", type = "JSJobChain", mandatory = false)
	public JSJobChain	scheduler_job_chain	= new JSJobChain(this, conClassName + ".scheduler_job_chain", // HashMap-Key
													"The name of a job chain for Managed File Transfer", // Titel
													"scheduler_sosftp_history", // InitValue
													"scheduler_sosftp_history", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getscheduler_job_chain
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public JSJobChain getscheduler_job_chain() {
		return scheduler_job_chain;
	}

	/**
	 * \brief setscheduler_job_chain
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_scheduler_job_chain
	 */
	public void setscheduler_job_chain(JSJobChain p_scheduler_job_chain) {
		this.scheduler_job_chain = p_scheduler_job_chain;
	}

	/**
	* \var scheduler_port : The port for which a Job Scheduler for Managed File Trans
	*
	The port for which a Job Scheduler for Managed File Transfer is operated, see parameter scheduler_host.
	*
	*/
	@JSOptionDefinition(name = "scheduler_port", description = "The port for which a Job Scheduler for Managed File Trans", key = "scheduler_port", type = "SOSOptionString", mandatory = false)
	public SOSOptionPortNumber	scheduler_port	= new SOSOptionPortNumber(this, conClassName + ".scheduler_port", // HashMap-Key
														"The port for which a Job Scheduler for Managed File Trans", // Titel
														"0", // InitValue
														"4444", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getscheduler_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionPortNumber getscheduler_port() {
		return scheduler_port;
	}

	/**
	 * \brief setscheduler_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_scheduler_port
	 */
	public void setscheduler_port(SOSOptionPortNumber p_scheduler_port) {
		this.scheduler_port = p_scheduler_port;
	}

	/**
	* \var settings : Name of INI-File which contains the profiles to execute
	*
	A configuration (INI-) file can be specified that contains profiles, i.e. sections, with parameters specified as pairs of names and values in a plain text format like this: [sample_transfer] protocol = ftp host = localhost port = 21 local_dir = /tmp ... At the command line the name of the configuration file and the profile are specified like this: sosftp.sh -operation=send -settings=settings.ini -profile=sample_transfer ... A profile can reference the contents of other profiles like this: [default] history = /sosftp/transfer_history.csv mandator = SOS scheduler_host = localhost scheduler_port = 4444 [sample_transfer] include = default protocol = ftp host = www.sos-berlin.com port = 21 local_dir = /tmp ... With this sample the profile sample_transfer includes the default profile via the include directive and thus applies the file transfer history settings.
	*
	*/
	@JSOptionDefinition(name = "settings", description = "Name of INI-File which contains the profiles to execute", key = "settings", type = "SOSOptionIniFileName", mandatory = false)
	public SOSOptionIniFileName	settings	= new SOSOptionIniFileName(this, conClassName + ".settings", // HashMap-Key
													"Name of INI-File which contains the profiles to execute", // Titel
													"", // InitValue
													"", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getsettings
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionIniFileName getsettings() {
		return settings;
	}

	/**
	 * \brief setsettings
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_settings
	 */
	public void setsettings(SOSOptionIniFileName p_settings) {
		this.settings = p_settings;
	}

	public SOSOptionIniFileName	SOSIniFileName	= (SOSOptionIniFileName) settings.SetAlias(conClassName + ".SOSIniFileName");

	/**
	* \var skip_transfer : If this Parameter is set to true then
	*
	If this Parameter is set to true then all operations except for the transfer itself will be performed. This can be used to just trigger for files or to only delete files on the FTP/SFTP server.
	*
	*/
	@JSOptionDefinition(name = "skip_transfer", description = "If this Parameter is set to true then", key = "skip_transfer", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean		skip_transfer	= new SOSOptionBoolean(this, conClassName + ".skip_transfer", // HashMap-Key
														"If this Parameter is set to true then", // Titel
														"false", // InitValue
														"false", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getskip_transfer
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean getskip_transfer() {
		return skip_transfer;
	}

	/**
	 * \brief setskip_transfer
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_skip_transfer
	 */
	public void setskip_transfer(SOSOptionBoolean p_skip_transfer) {
		this.skip_transfer = p_skip_transfer;
	}
 
	/**
	* \var ssh_auth_file : This parameter specifies the path and name of a us
	*
	This parameter specifies the path and name of a user's private key file that is used for authentication with an SSH server. This parameter has to be specified should the publickey authentication method have been specified in the ssh_auth_method parameter. Should the private key file be secured by a passphrase, then the passphrase has to be specified using the password parameter.
	*
	*/
	@JSOptionDefinition(name = "ssh_auth_file", description = "This parameter specifies the path and name of a us", key = "ssh_auth_file", type = "SOSOptionInFileName", mandatory = false)
	public SOSOptionInFileName	ssh_auth_file	= new SOSOptionInFileName(this, conClassName + ".ssh_auth_file", // HashMap-Key
														"This parameter specifies the path and name of a us", // Titel
														"", // InitValue
														"", // DefaultValue
														false // isMandatory
												);
	public SOSOptionInFileName			auth_file	= (SOSOptionInFileName) ssh_auth_file.SetAlias(conClassName + ".auth_file");

	/**
	 * \brief getssh_auth_file
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInFileName getssh_auth_file() {
		return ssh_auth_file;
	}

	/**
	 * \brief setssh_auth_file
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_ssh_auth_file
	 */
	public void setssh_auth_file(SOSOptionInFileName p_ssh_auth_file) {
		this.ssh_auth_file = p_ssh_auth_file;
	}

	/**
	* \var ssh_auth_method : This parameter specifies the authentication method
	*
	This parameter specifies the authentication method for the SSH server - the publickey and password methods are supported. Should the publickey authentication method be used, then the path name of the private key file has to be specified with the ssh_auth_file parameter. Should the private key file be secured by a passphrase then the passphrase has to be specified with the password parameter. For the password authentication method the password for the user account has to be specified using the password parameter. The authentication methods that are enabled depend on the SSH server configuration. Not all SSH servers are configured for password authentication.
	*
	*/
	@JSOptionDefinition(name = "ssh_auth_method", description = "This parameter specifies the authentication method", key = "ssh_auth_method", type = "SOSOptionStringValueList", mandatory = false)
	public SOSOptionAuthenticationMethod	ssh_auth_method	= new SOSOptionAuthenticationMethod(this, conClassName + ".ssh_auth_method", // HashMap-Key
																	"This parameter specifies the authentication method", // Titel
																	"publickey", // InitValue
																	"publickey", // DefaultValue
																	false // isMandatory
															);
	public SOSOptionAuthenticationMethod			auth_method	= (SOSOptionAuthenticationMethod) ssh_auth_method.SetAlias(conClassName + ".auth_method");

	/**
	 * \brief getssh_auth_method
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionAuthenticationMethod getssh_auth_method() {
		return ssh_auth_method;
	}

	/**
	 * \brief setssh_auth_method
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_ssh_auth_method
	 */
	public void setssh_auth_method(SOSOptionAuthenticationMethod p_ssh_auth_method) {
		this.ssh_auth_method = p_ssh_auth_method;
	}

	/**
	* \var ssh_proxy_host : The value of this parameter is the host name or th
	*
	The value of this parameter is the host name or the IP address of a proxy that is used in order to establish a connection to the SSH server. The use of a proxy is optional.
	*
	*/
	@JSOptionDefinition(name = "ssh_proxy_host", description = "The value of this parameter is the host name or th", key = "ssh_proxy_host", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	ssh_proxy_host	= new SOSOptionString(this, conClassName + ".ssh_proxy_host", // HashMap-Key
													"The value of this parameter is the host name or th", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getssh_proxy_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getssh_proxy_host() {
		return ssh_proxy_host;
	}

	/**
	 * \brief setssh_proxy_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_ssh_proxy_host
	 */
	public void setssh_proxy_host(SOSOptionString p_ssh_proxy_host) {
		this.ssh_proxy_host = p_ssh_proxy_host;
	}

	/**
	* \var ssh_proxy_password : This parameter specifies the password for the prox
	*
	This parameter specifies the password for the proxy server user account, should a proxy be used in order to connect to the SSH server.
	*
	*/
	@JSOptionDefinition(name = "ssh_proxy_password", description = "This parameter specifies the password for the prox", key = "ssh_proxy_password", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	ssh_proxy_password	= new SOSOptionString(this, conClassName + ".ssh_proxy_password", // HashMap-Key
														"This parameter specifies the password for the prox", // Titel
														" ", // InitValue
														" ", // DefaultValue
														false // isMandatory
												);

	/**
	 * \brief getssh_proxy_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getssh_proxy_password() {
		return ssh_proxy_password;
	}

	/**
	 * \brief setssh_proxy_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_ssh_proxy_password
	 */
	public void setssh_proxy_password(SOSOptionString p_ssh_proxy_password) {
		this.ssh_proxy_password = p_ssh_proxy_password;
	}

	/**
	* \var ssh_proxy_port : This parameter specifies the port number of the pr
	*
	This parameter specifies the port number of the proxy, should a proxy be used in order to establish a connection to the SSH server.
	*
	*/
	@JSOptionDefinition(name = "ssh_proxy_port", description = "This parameter specifies the port number of the pr", key = "ssh_proxy_port", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	ssh_proxy_port	= new SOSOptionString(this, conClassName + ".ssh_proxy_port", // HashMap-Key
													"This parameter specifies the port number of the pr", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getssh_proxy_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getssh_proxy_port() {
		return ssh_proxy_port;
	}

	/**
	 * \brief setssh_proxy_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_ssh_proxy_port
	 */
	public void setssh_proxy_port(SOSOptionString p_ssh_proxy_port) {
		this.ssh_proxy_port = p_ssh_proxy_port;
	}

	/**
	* \var ssh_proxy_user : The value of this parameter specifies the user acc
	*
	The value of this parameter specifies the user account for authentication by the proxy server should a proxy be used in order to connect to the SSH server.
	*
	*/
	@JSOptionDefinition(name = "ssh_proxy_user", description = "The value of this parameter specifies the user acc", key = "ssh_proxy_user", type = "SOSOptionString", mandatory = false)
	public SOSOptionString	ssh_proxy_user	= new SOSOptionString(this, conClassName + ".ssh_proxy_user", // HashMap-Key
													"The value of this parameter specifies the user acc", // Titel
													" ", // InitValue
													" ", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief getssh_proxy_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionString getssh_proxy_user() {
		return ssh_proxy_user;
	}

	/**
	 * \brief setssh_proxy_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_ssh_proxy_user
	 */
	public void setssh_proxy_user(SOSOptionString p_ssh_proxy_user) {
		this.ssh_proxy_user = p_ssh_proxy_user;
	}

	/**
	* \var transactional : This parameter specifies if file transfers should
	*
	This parameter specifies if file transfers should be operated within a single transaction, i.e. either all files are successfully transferred or none. Should an error occur during a transfer operation then all transfers will be rolled back. When specifying the value true then the following applies: The parameter atomic_suffix has to be specified that causes target files to be created with a suffix such as "~" and that causes the respective files to be renamed to their target file name after the transfer of all files has been successfully completed. If at least one file out of a set of files cannot be transferred successfully then no files will be renamed, instead the temporarily created files are removed from the target system. The parameter remove_files that causes files to be removed after successful transfer will be effective only after all files have been successfully transferred. Otherwise no files will be removed.
	*
	*/
	@JSOptionDefinition(name = "transactional", description = "This parameter specifies if file transfers should", key = "transactional", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean	transactional	= new SOSOptionBoolean(this, conClassName + ".transactional", // HashMap-Key
													"This parameter specifies if file transfers should", // Titel
													"false", // InitValue
													"false", // DefaultValue
													false // isMandatory
											);

	/**
	 * \brief gettransactional
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionBoolean gettransactional() {
		return transactional;
	}

	/**
	 * \brief settransactional
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_transactional
	 */
	public void settransactional(SOSOptionBoolean p_transactional) {
		this.transactional = p_transactional;
	}

	public SOSOptionBoolean			TransactionMode	= (SOSOptionBoolean) transactional.SetAlias(conClassName + ".TransactionMode");

	/**
	* \var transfer_mode : Type of Character-Encoding Transfe
	*
	Transfer mode is used for FTP exclusively and can be either ascii or binary.
	*
	*/
	@JSOptionDefinition(name = "transfer_mode", description = "Type of Character-Encoding Transfe", key = "transfer_mode", type = "SOSOptionTransferMode", mandatory = false)
	public SOSOptionTransferMode	transfer_mode	= new SOSOptionTransferMode(this, conClassName + ".transfer_mode", // HashMap-Key
															"Type of Character-Encoding Transfe", // Titel
															"binary", // InitValue
															"binary", // DefaultValue
															false // isMandatory
													);

	/**
	 * \brief gettransfer_mode
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionTransferMode gettransfer_mode() {
		return transfer_mode;
	}

	/**
	 * \brief settransfer_mode
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_transfer_mode
	 */
	public void settransfer_mode(SOSOptionTransferMode p_transfer_mode) {
		this.transfer_mode = p_transfer_mode;
	}

	/**
	* \var user : UserID of user in charge User name
	*
	User name for authentication at the (FTP/SFTP) server.
	*
	*/
	@JSOptionDefinition(name = "user", description = "UserID of user in charge User name", key = "user", type = "SOSOptionUserName", mandatory = true)
	public SOSOptionUserName	user	= new SOSOptionUserName(this, conClassName + ".user", // HashMap-Key
												"UserID of user in charge User name", // Titel
												"", // InitValue
												"anonymous", // DefaultValue
												false // isMandatory
										);

	/**
	 * \brief getuser
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionUserName getuser() {
		return user;
	}

	/**
	 * \brief setuser
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_user
	 */
	public void setuser(SOSOptionUserName p_user) {
		this.user = p_user;
	}

	/**
	* \var verbose : The granuality of (Debug-)Messages The verbosit
	*
	The verbosity level specifies the intensity of log entries. A value between 1 and 9 can be specified. Higher values cause more detailed information to be logged. Log output is written to stdout or to a file that has been specified with the parameter log_filename.
	*
	*/
	@JSOptionDefinition(name = "verbose", description = "The granuality of (Debug-)Messages The verbosit", key = "verbose", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger	verbose	= new SOSOptionInteger(this, conClassName + ".verbose", // HashMap-Key
											"The granuality of (Debug-)Messages The verbosit", // Titel
											"1", // InitValue
											"10", // DefaultValue
											false // isMandatory
									);

	/**
	 * \brief getverbose
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionInteger getverbose() {
		return verbose;
	}

	/**
	 * \brief setverbose
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_verbose
	 */
	public void setverbose(SOSOptionInteger p_verbose) {
		this.verbose = p_verbose;
	}

	public SOSOptionInteger			VerbosityLevel		= (SOSOptionInteger) verbose.SetAlias(conClassName + ".VerbosityLevel");

	/**
	* \var zero_byte_transfer : This parameter specifies whether zero byte files
	*
	This parameter specifies whether zero byte files should be transferred and processed by subsequent commands. The following settings are available: yes : Files with zero byte size are transferred (default). no : Files with zero byte size are transferred, should at least one of the files have more than zero byte size. strict : Files with zero byte size are not transferred. An error will be raised if any zero byte file is found. relaxed : Files with zero byte size will not be transferred. However, no error will be raised if this results in no files being transferred. Use of this parameter can be refined using the force_files parameter: should force_files have the value false, then processing will be treated as successful in the event of no files having been transferred. Note that the remove_files parameter has unrestricted validity. Files with zero byte size will be removed regardless of whether or not they have been transferred.
	*
	*/
	@JSOptionDefinition(name = "zero_byte_transfer", description = "This parameter specifies whether zero byte files", key = "zero_byte_transfer", type = "SOSOptionStringValueList", mandatory = false)
	public SOSOptionStringValueList	zero_byte_transfer	= new SOSOptionStringValueList(this, conClassName + ".zero_byte_transfer", // HashMap-Key
																"This parameter specifies whether zero byte files", // Titel
																"yes", // InitValue
																"yes", // DefaultValue
																false // isMandatory
														);

	/**
	 * \brief getzero_byte_transfer
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	public SOSOptionStringValueList getzero_byte_transfer() {
		return zero_byte_transfer;
	}

	/**
	 * \brief setzero_byte_transfer
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param p_zero_byte_transfer
	 */
	public void setzero_byte_transfer(SOSOptionStringValueList p_zero_byte_transfer) {
		this.zero_byte_transfer = p_zero_byte_transfer;
	}

	public SOSFtpOptionsSuperClass() {
//		super("SOSVirtualFileSystem");
		objParentClass = this.getClass();
	} // public SOSFtpOptionsSuperClass

	public SOSFtpOptionsSuperClass(JSListener pobjListener) {
		this();
		this.registerMessageListener(pobjListener);
	} // public SOSFtpOptionsSuperClass

	//

	public SOSFtpOptionsSuperClass(HashMap<String, String> JSSettings) throws Exception {
		this();
		this.setAllOptions(JSSettings);
	} // public SOSFtpOptionsSuperClass (HashMap JSSettings)

	//
	/**
	* \brief getAllOptionsAsString - liefert die Werte und Beschreibung aller
	* Optionen als String
	*
	* \details
	*
	* \see toString \see toOut
	*/
	@SuppressWarnings("unused")
	private String getAllOptionsAsString() {
		final String conMethodName = conClassName + "::getAllOptionsAsString";
		String strT = conClassName + "\n";
		final StringBuffer strBuffer = new StringBuffer();
		// strT += IterateAllDataElementsByAnnotation(objParentClass, this,
		// JSOptionsClass.IterationTypes.toString, strBuffer);
		// strT += IterateAllDataElementsByAnnotation(objParentClass, this, 13,
		// strBuffer);
		strT += this.toString(); // fix
		//
		return strT;
	} // private String getAllOptionsAsString ()

	/**
	 * \brief setAllOptions
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param pobjJSSettings
	 * @throws Exception
	 */
	public void setAllOptions(HashMap<String, String> pobjJSSettings) throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setAllOptions";
		flgSetAllOptions = true;
		objSettings = pobjJSSettings;
		super.Settings(objSettings);
		super.setAllOptions(pobjJSSettings);
		flgSetAllOptions = false;
	} // public void setAllOptions (HashMap <String, String> JSSettings)

	//

	/**
	 * \brief CheckMandatory
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @throws JSExceptionMandatoryOptionMissing
	 */
	@Override
	public void CheckMandatory() throws com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing //
	{
		try {
			super.CheckMandatory();
		}
		catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
	} // public void CheckMandatory ()

	//

	/**
	 * \brief CommandLineArgs
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param pstrArgs
	 * @throws Exception
	 */
	@Override
	public void CommandLineArgs(String[] pstrArgs) throws Exception {
		super.CommandLineArgs(pstrArgs);
		this.setAllOptions(super.objSettings);
	}

	/**
	 * \brief getHost
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionHostName getHost() {

		return host;
	}

	/**
	 * \brief getPort
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionPortNumber getPort() {
		return port;
	}

	/**
	 * \brief getProxy_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getProxy_host() {
		return null;
		// return proxy_host;
	}

	/**
	 * \brief getProxy_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionPassword getProxy_password() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * \brief getProxy_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionPortNumber getProxy_port() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * \brief getProxy_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getProxy_user() {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * \brief setHost
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param host
	 */
	@Override
	public void setHost(SOSOptionHostName host) {
		this.sethost(host);
	}

	/**
	 * \brief setPort
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param port
	 */
	@Override
	public void setPort(SOSOptionPortNumber port) {
		// TODO Auto-generated method stub

	}

	/**
	 * \brief setProxy_host
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param proxyHost
	 */
	@Override
	public void setProxy_host(SOSOptionString proxyHost) {
		// TODO Auto-generated method stub

	}

	/**
	 * \brief setProxy_password
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param proxyPassword
	 */
	@Override
	public void setProxy_password(SOSOptionPassword proxyPassword) {
		// TODO Auto-generated method stub

	}

	/**
	 * \brief setProxy_port
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param proxyPort
	 */
	@Override
	public void setProxy_port(SOSOptionPortNumber proxyPort) {
		// TODO Auto-generated method stub

	}

	/**
	 * \brief setProxy_user
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param proxyUser
	 */
	@Override
	public void setProxy_user(SOSOptionString proxyUser) {
		// TODO Auto-generated method stub

	}

	/**
	 * \brief getAuth_file
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionInFileName getAuth_file() {
		return ssh_auth_file;
	}

	/**
	 * \brief getAuth_method
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionAuthenticationMethod getAuth_method() {
		return ssh_auth_method;
	}

	/**
	 * \brief getPassword
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionPassword getPassword() {
		return password;
	}

	/**
	 * \brief getUser
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public SOSOptionString getUser() {
		return user;
	}

	/**
	 * \brief setAuth_file
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param authFile
	 */
	@Override
	public void setAuth_file(SOSOptionInFileName authFile) {
		this.ssh_auth_file = authFile;
	}

	/**
	 * \brief setAuth_method
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param authMethod
	 */
	@Override
	public void setAuth_method(SOSOptionAuthenticationMethod authMethod) {
		// TODO Auto-generated method stub

	}

	/**
	 * \brief setPassword
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param password
	 */
	@Override
	public void setPassword(SOSOptionPassword password) {
		// TODO Auto-generated method stub

	}

	/**
	 * \brief setUser
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param pobjUser
	 */
	@Override
	public void setUser(SOSOptionString pobjUser) {
		// TODO Auto-generated method stub
		this.user.Value(pobjUser.Value());
	}

} // public class SOSFtpOptionsSuperClass
//

