/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.dailyschedule.dialog.classes;

import java.util.prefs.Preferences;

import com.sos.dialog.classes.FormBase;
import com.sos.dialog.comparators.DateComperator;
import com.sos.dialog.comparators.SortBaseComparator;
import com.sos.dialog.comparators.StringComparator;
import com.sos.dialog.interfaces.ITableView;
import com.sos.hibernate.classes.DbItem;
import com.sos.hibernate.classes.SosSortTableItem;
import com.sos.hibernate.interfaces.ISOSDashboardDataProvider;
import com.sos.hibernate.interfaces.ISOSTableItem;
import com.sos.scheduler.history.SchedulerHistoryDataProvider;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
 
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
 
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Listener;

public class SOSDashboardTableView extends FormBase implements ITableView {

    
    private static final String             conSettingREFRESHDefault            = "60";
    private static final String             SOS_DASHBOARD_HEADER                = "sosDashboardHeader";
    private static final String             conSettingREFRESH                   = "refresh";
    private static final int                NUMBER_OF_COLUMNS_IN_GRID           = 9;

    final String conSOSDashB_show_log_in_new_tab = "SOSDashB_show_log_in_new_tab"; // "Show log in a new tab";
    final String conSOSDashB_only_stand_alone_jobs = "SOSDashB_only_stand_alone_jobs"; // "Only stand alone jobs";
    final String conSOSDashB_only_job_chains = "SOSDashB_only_job_chains"; // "Only job chains";
    final String conSOSDashB_only_late = "SOSDashB_only_late"; // "Only late";
    final String conSOSDashB_only_waiting = "SOSDashB_only_waiting"; // "Only waiting";
    final String conSOSDashB_start_now = "SOSDashB_start_now"; // "Start now";
    final String conSOSDashB_Today = "SOSDashB_Today"; // "Today";
    final String conSOSDashB_Reset = "SOSDashB_Reset"; // "Reset";
    final String conSOSDashB_only_executed = "SOSDashB_only_executed"; // "only executed"

 
    protected Group right;
    protected Group left;

    protected CTabFolder leftTabFolder = null;
    protected CTabFolder logTabFolder = null;

    protected Preferences prefs;

    private static Logger logger = Logger.getLogger(SOSDashboardTableView.class);
    protected Table tableList = null;
    protected ISOSDashboardDataProvider tableDataProvider = null;
    protected SchedulerHistoryDataProvider detailHistoryDataProvider = null;
    protected SosDashboardHeader sosDashboardHeader = null;
    private SortBaseComparator[][] comparables = null;
    protected Composite tableComposite = null;
    protected Table tableHistoryDetail = null;
    protected SOSDashboardOptions objOptions = null;
     
    public SOSDashboardTableView(Composite composite_) {
        super(composite_, "SosSchedulerDashboardMain");
    }

    @Override
    public void getTableData() {
        logger.debug("...getList");

        if (tableList != null) {
            tableDataProvider.getData();
            buildTable();
        }
    }

    @Override
    public void buildTable() {
        logger.debug("Building table Planned");
        this.showWaitCursor();
        if (tableList != null) {

            left.setText(tableDataProvider.getFilter().getTitle());
            clearTable(tableList);
            tableDataProvider.fillTable(tableList);
            tableDataProvider.fillSchedulerIds(sosDashboardHeader.getCbSchedulerId());

            SosSortTableItem sosSortTableItem = null;

            sosSortTableItem = null;
            int ll = tableList.getItemCount();
            comparables = new SortBaseComparator[tableList.getColumnCount()][ll];

            for (int i = 0; i < tableList.getItemCount(); i++) {
                for (int k = 0; k < tableList.getColumnCount(); k++) {
                    sosSortTableItem = new SosSortTableItem((ISOSTableItem) tableList.getItems()[i]);
                    if (k == 4 || k == 5) {
                        comparables[k][i] = new DateComperator(sosSortTableItem, i, k);
                    } else {
                        comparables[k][i] = new StringComparator(sosSortTableItem, i, k);
                    }
                }
            }
            sortTable(tableList, 4, comparables, false);

        }
        this.RestoreCursor();

    }
    
    @Override
    public void createTable() {
        tableComposite = new Composite(leftTabFolder, SWT.NONE);

        GridLayout layout = new GridLayout(NUMBER_OF_COLUMNS_IN_GRID, false);
        tableComposite.setLayout(layout);

        sosDashboardHeader = new SosDashboardHeader(tableComposite, this);
        sosDashboardHeader.getCbSchedulerId().addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                tableDataProvider.setSchedulerId(sosDashboardHeader.getCbSchedulerId().getText());
                getList();
            }
        });

     
        
        sosDashboardHeader.getToDate().addSelectionListener (new SelectionAdapter () {
            public void widgetSelected (SelectionEvent e) {
                tableDataProvider.setTo(sosDashboardHeader.getTo());
                getList();
            }
          });
        sosDashboardHeader.getFromDate().addSelectionListener (new SelectionAdapter () {
            public void widgetSelected (SelectionEvent e) {
                tableDataProvider.setFrom(sosDashboardHeader.getFrom());
                getList();
            }
          });


        sosDashboardHeader.getRefreshInterval().setText(prefs.node(SOS_DASHBOARD_HEADER).get(conSettingREFRESH, conSettingREFRESHDefault));
        sosDashboardHeader.getRefreshInterval().addModifyListener(new ModifyListener() {
            public void modifyText(final ModifyEvent arg0) {
                sosDashboardHeader.setRefresh(getIntValue(sosDashboardHeader.getRefreshInterval().getText(), 10));
                sosDashboardHeader.resetRefreshTimer();
                prefs.node(SOS_DASHBOARD_HEADER).put(conSettingREFRESH, sosDashboardHeader.getRefreshInterval().getText());
            }
        });

        sosDashboardHeader.getSearchField().addModifyListener(new ModifyListener() {
            public void modifyText(final ModifyEvent e) {
                if (sosDashboardHeader.getSearchField() != null) {
                    tableDataProvider.setSearchField(sosDashboardHeader.getSearchField().getText());
                    sosDashboardHeader.resetInputTimer();
                }
            }
        });

        sosDashboardHeader.getRefreshButton().addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(final SelectionEvent e) {
                getList();
            }
        });

        tableList = new SosExecutedTable(tableComposite, SWT.FULL_SELECTION | SWT.BORDER);
        tableList.addListener(SWT.MouseDown, new Listener() {
            public void handleEvent(Event event) {
                setRightMausclick(event);
            }
        });

        tableList.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(final SelectionEvent e) {
                if (!isRightMouseclick()) {

                    TableItem t = tableList.getItem(tableList.getSelectionIndex());

                    showLog(tableList);

                    DbItem d = (DbItem) t.getData();
                    right.setText(d.getTitle());

                    detailHistoryDataProvider.setFrom(sosDashboardHeader.getFrom());
                    detailHistoryDataProvider.setTo(sosDashboardHeader.getTo());
                    detailHistoryDataProvider.setSchedulerId(d.getSchedulerId());
                    detailHistoryDataProvider.setJobname(d.getJob());
                    detailHistoryDataProvider.setJobchain(d.getJobChain());
                    detailHistoryDataProvider.setOrderid(d.getOrderId());
                    detailHistoryDataProvider.getData();
                    clearTable(tableHistoryDetail);
                    detailHistoryDataProvider.fillTableShort(tableHistoryDetail, d.isStandalone());

                }
            }
        });

        this.setColumnsListener();
        
        
    }

    public void setColumnsListener(){
        TableColumn[] columns = tableList.getColumns();
         for (int i = 0; i < columns.length; i++) {
            final int _i = i;
            columns[i].addListener(SWT.Selection, new Listener() {
                private int colPos = -1;
                private boolean sortFlag;
                {
                    colPos = _i;
                }

                public void handleEvent(Event event) {
                    sortFlag = !sortFlag;
                    tableList.setSortColumn(tableList.getColumn(colPos));
                    if (sortFlag) {
                        tableList.setSortDirection(SWT.UP);
                    } else {
                        tableList.setSortDirection(SWT.DOWN);
                    }

                    sortTable(tableList, colPos, comparables, sortFlag);
                }

            });
        }
    }
    
  
  

    protected void showLog(Table table) {
        this.showWaitCursor();
        if (table.getSelectionIndex() >= 0 && table.getSelectionIndex() >= 0) {

            SosTabLogItem logItem = (SosTabLogItem) logTabFolder.getSelection();
            if (logItem == null) {
                logTabFolder.setSelection(0);
                logItem = (SosTabLogItem) logTabFolder.getSelection();
            }

            TableItem t = table.getItem(table.getSelectionIndex());
            DbItem d = (DbItem) t.getData();

            logItem.addLog(table, d.getTitle(), detailHistoryDataProvider.getLogAsString(d));

        }
        this.RestoreCursor();

    }

    

    @Override
    public void getList() {
        logger.debug("...getList");

        if (tableList != null) {
            tableDataProvider.getData();

            buildTable();
        }

    }

    @Override
    public void actualizeList() {
        buildTable();
    }

    

    public void setLeftTabFolder(CTabFolder leftTabFolder) {
        this.leftTabFolder = leftTabFolder;
    }

    public void setPrefs(Preferences prefs) {
        this.prefs = prefs;
    }

     
  
    public void setDetailHistoryDataProvider(SchedulerHistoryDataProvider detailHistoryDataProvider) {
        this.detailHistoryDataProvider = detailHistoryDataProvider;
    }

    public void setSosDashboardHeaderplanned(SosDashboardHeader sosDashboardHeader) {
        this.sosDashboardHeader = sosDashboardHeader;
    }

    public void setComparablesplanned(SortBaseComparator[][] comparablesplanned) {
        this.comparables = comparablesplanned;
    }

    public void setLogTabFolder(CTabFolder logTabFolder) {
        this.logTabFolder = logTabFolder;
    }

    public void setTableHistoryDetail(Table tableHistoryDetail) {
        this.tableHistoryDetail = tableHistoryDetail;
    }

    public Composite getTablePlannedComposite() {
        return tableComposite;
    }

    public SosDashboardHeader getSosDashboardHeader() {
        return sosDashboardHeader;
    }

    public void setObjOptions(SOSDashboardOptions objOptions) {
        this.objOptions = objOptions;
    }
 

    @Override
    public void createMenue() {
       logger.info("No menu is defined");        
    }

    public void setRight(Group right) {
        this.right = right;
    }

    public void setLeft(Group left) {
        this.left = left;
    }

    public Composite getTableComposite() {
        return tableComposite;
    }

    public void setTableDataProvider(ISOSDashboardDataProvider tableDataProvider) {
        this.tableDataProvider = tableDataProvider;
    }
 

}
