/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.model.objects;

import java.util.Date;

import org.joda.time.DateTime;
import org.joda.time.Interval;

public interface ISOSJsObjPeriod {

	public abstract PeriodType getPeriodType();

	/**
	 * \brief detects if a period is a simple single start
	 * @return
	 */
	public abstract boolean hasSingleStart();

	/**
	 * \brief detects if a period is an interval with a start to start relation
	 * @return
	 */
	public abstract boolean hasStartStartInterval();

	/**
	 * \brief detects if a period is an interval with a end to start relation
	 * @return
	 */
	public abstract boolean hasEndStartInterval();

	/**
	 * \brief ensures that the begin of the period has the format hh:mm:ss
	 * @see com.sos.scheduler.model.objects.Period#getBegin()
	 */
	public abstract String getBegin();

	/**
	 * \brief ensures that the end of the period has the format hh:mm:ss
	 * @see com.sos.scheduler.model.objects.Period#getEnd()
	 */
	public abstract String getEnd();

	/**
	 * \brief ensures that the singel start time has the format hh:mm:ss
	 * @see com.sos.scheduler.model.objects.Period#getSinlgeStart()
	 */
	public abstract String getSingleStart();

	public abstract boolean isInPeriod(Date timeStamp);

	public abstract boolean isInPeriod(DateTime timeStamp);

	public abstract Interval getPeriod(DateTime date);

	/**
	 * \brief get the single start as a DateTime object
	 * \detail
	 * To calculate the starttime it is necessary to call this method with a base date.
	 * @param date
	 * @return
	 */
	public DateTime getDtSingleStart(DateTime date);
	
	public DateTime getDtBegin(DateTime date);
	
	public DateTime getDtEnd(DateTime date);

}
