/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package sos.scheduler.command;



import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.IOException;


import java.net.*;


/**
 * <p>Title: </p>
 * <p>Description: this class represents a simple client for the scheduler</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: SOS GmbH</p>
 * @author <a href="mailto:ghassan.beydoun@sos-berlin.com">Ghassan Beydoun</a>
 * @version $Id: SOSSchedulerCommand.java 15153 2011-09-14 11:59:34Z kb $
 */

public class SOSSchedulerCommand {

  /** host */
  private String host = "localhost";

  /** port: default is 44444 */
  private int port    = 4444;

  /** protocl: default is tcp */
  private String protocol = "tcp";


  private Socket socket = null;

  private DatagramSocket udpSocket = null; 
  
  /** timeout fr getResponse in sekunden: default ist 5 sek. */
  private int timeout = 5;


  private BufferedReader in = null;

  private PrintWriter out = null;


  public SOSSchedulerCommand() {}
  
  public SOSSchedulerCommand(String host) {
      
      this.setHost(host);
  }
  
  public SOSSchedulerCommand(String host, int port) {
      
      this.setHost(host);
      this.setPort(port);
  }
  
  /**
 * @param host the hostname (or ip address) of the Job Scheduler
 * @param port the port of the Job Scheduler
 * @param protocol the connection protocol ("tcp" or "udp")
 */
public SOSSchedulerCommand(String host, int port, String protocol) {
      
      this.setHost(host);
      this.setPort(port);
      this.setProtocol(protocol);
  }
  
  
  
  
  /**
   * Sets the hostname (or ip address) of the Job Scheduler
   * @param host
   */
  public void setHost( String host ) {
    this.host = host;
  }


  public String getHost() {
      return this.host;
    }

  /**
   * Sets the port of the Job Scheduler
   * @param port
   */
  public void setPort( int port ) {
    this.port = port;
  }



  public int getPort() {
    return this.port;
  }

  /**
   * @param timeout
   */
  public void setTimeout( int timeout ) {
    this.timeout = timeout;
  }


  /**
   * 
   * @return Returns the protocol.
   */
  public String getProtocol() {
      return protocol;
  }
  /**
   * @param protocol sets the protocol to "tcp" or "udp"
   */
  public void setProtocol(String protocol) {
      this.protocol = protocol;
  }

  

  
  /**
   * connects to the scheduler
   *
   * @param host Job Scheduler host
   * @param port Job Scheduler port
   * @throws java.lang.Exception
   */
  public void connect( String host, int port ) throws Exception {

      if( host == null || host.length() == 0 )
      throw (new Exception("hostname missing."));

    if( port == 0 )
      throw (new Exception("port missing."));

    if (this.protocol.equalsIgnoreCase("udp")) {

        udpSocket = new DatagramSocket();
        udpSocket.connect(InetAddress.getByName(this.host), this.port);
        
    } else {

        socket = new Socket(host, port);
        if (this.getTimeout() > 0) socket.setSoTimeout(this.getTimeout()*1000);
        in     = new BufferedReader( new InputStreamReader(socket.getInputStream()));
        out    = new PrintWriter(socket.getOutputStream(), true);
    }
  }


  /**
   * connects to the scheduler
   *
   * @throws java.lang.Exception
   */
  public void connect() throws Exception {
    this.connect(this.host,this.port);
  }



  /**
   * sends a command to the scheduler
   *
   * @param command XML String containing the command
   * @throws java.lang.Exception
   */
  public void sendRequest( String command ) throws Exception {

      if ( this.protocol.equalsIgnoreCase("udp")) {
          if ( command.indexOf("<?xml") == -1 ) {
              command = "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>" + command + "\r\n";
          }
          byte[] commandBytes = command.getBytes();
          udpSocket.send(new DatagramPacket(commandBytes, commandBytes.length, InetAddress.getByName(this.host), this.port));
	
      } else {
          if ( command.indexOf("<?xml") == 0 ) {
              out.print(command + "\r\n");
          } else {
              out.print( "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>" + command + "\r\n" );
          }
          out.flush();
      }
  }


  /**
   * returns the scheduler reply.
   *
   * @return String represents the reply of the scheduler on success, 
   * otherwise empty string
   * @throws IOException
   * @throws RuntimeException
   */
  public String getResponse() throws IOException,RuntimeException {
  	
      int b;
      StringBuffer response = new StringBuffer();

      if ( in != null) {
        while(  (b = in.read()) != -1 ) {
          if ( b == 0)
              break;
          response.append((char)b);

        } // while
      }
      return response.toString();
  }


  
  /**
 * @param host Job Scheduler host
 * @param port Job Scheduler port
 * @param xmlCommand XML String containing the command
 * @throws Exception
 */
public static void sendCommand( String host, int port, String xmlCommand ) throws Exception {

      SOSSchedulerCommand command = null;

      try {
          command = new SOSSchedulerCommand();
  
          command.setHost(host);
          command.setPort(port);
          command.setProtocol("udp");

          command.connect();
          command.sendRequest(xmlCommand);
  
      } catch (Exception e) {
          throw new Exception("startJob: could not start job: " + e.getMessage());
      }
  }
  

public static void addOrder( String host, int port, int status, String jobChain ) throws Exception {
	sendCommand(host,port,"<add_order job_chain=\"" + jobChain + "\" state=\"" + status + "\">"
            + "<params></params></add_order>");
}

public static void startJob( String host, int port, String job ) throws Exception {
	sendCommand(host,port,"<job job=\"" + job + "\">");
}




  /**
   * close the connection
   *
   * @throws java.lang.Exception
   */
  public void disconnect() throws Exception {
    if (socket != null) socket.close();
    if (in != null) in.close();
    if (out != null) out.close();
  }


  public static void main(String[] args) throws Exception {

    int argc=0, i=0;
    final String CLASS_NAME = "SOSSchedulerCommand";

    final String USAGE = "\nUsage: java " + CLASS_NAME +
        " [ -host <host> -port <port> ] <xml-command>";

    boolean params = false;
    String host    = null;
    String command = null;
    int port       = 0;

    argc = args.length;

    if ( argc == 5 ) {
      if ( args[0].equals("-host") ) {
        i++;
        host = args[i++];
        if( args[i++].equals("-port") ) {
          port = Integer.parseInt(args[i++]);
        } else {
          System.out.println(USAGE);
          System.exit(0);
        }
        command = args[i];
      } else {
        System.out.println(USAGE);
        System.exit(0);
      }
    } else if (argc == 1) { // default parameter verwenden
      params = true;
    } else {
      System.out.println(USAGE);
      System.exit(0);
    }

    
    
    SOSSchedulerCommand socket = null;
    try {
      socket = new SOSSchedulerCommand();
      
      
      if (params)
        socket.connect();
      else
        socket.connect(host, port); // sag,4371sag
        //c.sendRequest("<show_state what=\"all\"/>");
      socket.sendRequest(command);
      System.out.println(socket.getResponse());

    } catch(Exception e) {
      System.out.println(e.getMessage());
      System.out.println(USAGE);
    } finally {
      if( socket != null) socket.disconnect();
      System.exit(0);
    }

  }

/**
 * @return the timeout
 */
public int getTimeout() {
    return timeout;
}


}
